#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Database API

Originally based on web.py http://webpy.org/
"""

__all__ = [
    "UnknownParamstyle", "UnknownDB", "TransactionError",
    "sqllist", "sqlors", "reparam", "sqlquote",
    "SQLQuery", "SQLParam", "sqlparam",
    "SQLLiteral", "sqlliteral",
    "database", 'DB',
]

import time
import os
import urllib
import itertools
import sys

try:
    import datetime
except ImportError:
    datetime = None

from torngas.utils import IterBetter
from torngas.utils import safeunicode, safestr
from torngas.storage import Storage, ThreadedDict

try:
    set
except NameError:
    from sets import Set as set

from threading import local as threadlocal

debug = sys.stderr

iters = [list, tuple]
import __builtin__

if hasattr(__builtin__, 'set'):
    iters.append(set)
if hasattr(__builtin__, 'frozenset'):
    iters.append(set)
if sys.version_info < (2, 6): # sets module deprecated in 2.6
    try:
        from sets import Set

        iters.append(Set)
    except ImportError:
        pass


class _hack(tuple): pass


iters = _hack(iters)
iters.__doc__ = """
A list of iterable items (like lists, but not strings). Includes whichever
of lists, tuples, sets, and Sets are available in this version of Python.
"""

iterbetter = IterBetter
# for backward-compatibility
utf8 = safestr
storage = Storage
config = Storage()
threadeddict = ThreadedDict


class UnknownDB(Exception):
    """raised for unsupported dbms"""
    pass


class _ItplError(ValueError):
    def __init__(self, text, pos):
        ValueError.__init__(self)
        self.text = text
        self.pos = pos

    def __str__(self):
        return "unfinished expression in %s at char %d" % (
            repr(self.text), self.pos)


class TransactionError(Exception): pass


class UnknownParamstyle(Exception):
    """
    raised for unsupported db paramstyles

    (currently supported: qmark, numeric, format, pyformat)
    """
    pass


class SQLParam(object):
    """
    Parameter in SQLQuery.

        >>> q = SQLQuery(["SELECT * FROM test WHERE name=", SQLParam("joe")])
        >>> q
        <sql: "SELECT * FROM test WHERE name='joe'">
        >>> q.query()
        'SELECT * FROM test WHERE name=%s'
        >>> q.values()
        ['joe']
    """
    __slots__ = ["value"]

    def __init__(self, value):
        self.value = value

    def get_marker(self, paramstyle='pyformat'):
        if paramstyle == 'qmark':
            return '?'
        elif paramstyle == 'numeric':
            return ':1'
        elif paramstyle is None or paramstyle in ['format', 'pyformat']:
            return '%s'
        raise UnknownParamstyle, paramstyle

    def sqlquery(self):
        return SQLQuery([self])

    def __add__(self, other):
        return self.sqlquery() + other

    def __radd__(self, other):
        return other + self.sqlquery()

    def __str__(self):
        return str(self.value)

    def __repr__(self):
        return '<param: %s>' % repr(self.value)


sqlparam = SQLParam


class SQLQuery(object):
    """
    You can pass this sort of thing as a clause in any db function.
    Otherwise, you can pass a dictionary to the keyword argument `vars`
    and the function will call reparam for you.

    Internally, consists of `items`, which is a list of strings and
    SQLParams, which get concatenated to produce the actual query.
    """
    __slots__ = ["items"]

    # tested in sqlquote's docstring
    def __init__(self, items=None):
        r"""Creates a new SQLQuery.

            >>> SQLQuery("x")
            <sql: 'x'>
            >>> q = SQLQuery(['SELECT * FROM ', 'test', ' WHERE x=', SQLParam(1)])
            >>> q
            <sql: 'SELECT * FROM test WHERE x=1'>
            >>> q.query(), q.values()
            ('SELECT * FROM test WHERE x=%s', [1])
            >>> SQLQuery(SQLParam(1))
            <sql: '1'>
        """
        if items is None:
            self.items = []
        elif isinstance(items, list):
            self.items = items
        elif isinstance(items, SQLParam):
            self.items = [items]
        elif isinstance(items, SQLQuery):
            self.items = list(items.items)
        else:
            self.items = [items]

        # Take care of SQLLiterals
        for i, item in enumerate(self.items):
            if isinstance(item, SQLParam) and isinstance(item.value, SQLLiteral):
                self.items[i] = item.value.v

    def append(self, value):
        self.items.append(value)

    def __add__(self, other):
        if isinstance(other, basestring):
            items = [other]
        elif isinstance(other, SQLQuery):
            items = other.items
        else:
            return NotImplemented
        return SQLQuery(self.items + items)

    def __radd__(self, other):
        if isinstance(other, basestring):
            items = [other]
        else:
            return NotImplemented

        return SQLQuery(items + self.items)

    def __iadd__(self, other):
        if isinstance(other, (basestring, SQLParam)):
            self.items.append(other)
        elif isinstance(other, SQLQuery):
            self.items.extend(other.items)
        else:
            return NotImplemented
        return self

    def __len__(self):
        return len(self.query())

    def query(self, paramstyle=None):
        """
        Returns the query part of the sql query.
            >>> q = SQLQuery(["SELECT * FROM test WHERE name=", SQLParam('joe')])
            >>> q.query()
            'SELECT * FROM test WHERE name=%s'
            >>> q.query(paramstyle='qmark')
            'SELECT * FROM test WHERE name=?'
        """
        s = []
        for x in self.items:
            if isinstance(x, SQLParam):
                x = x.get_marker(paramstyle)
                s.append(safestr(x))
            else:
                x = safestr(x)
                # automatically escape % characters in the query
                # For backward compatability, ignore escaping when the query looks already escaped
                if paramstyle in ['format', 'pyformat']:
                    if '%' in x and '%%' not in x:
                        x = x.replace('%', '%%')
                s.append(x)
        return "".join(s)

    def values(self):
        """
        Returns the values of the parameters used in the sql query.
            >>> q = SQLQuery(["SELECT * FROM test WHERE name=", SQLParam('joe')])
            >>> q.values()
            ['joe']
        """
        return [i.value for i in self.items if isinstance(i, SQLParam)]

    def join(items, sep=' ', prefix=None, suffix=None, target=None):
        """
        Joins multiple queries.

        >>> SQLQuery.join(['a', 'b'], ', ')
        <sql: 'a, b'>

        Optinally, prefix and suffix arguments can be provided.

        >>> SQLQuery.join(['a', 'b'], ', ', prefix='(', suffix=')')
        <sql: '(a, b)'>

        If target argument is provided, the items are appended to target instead of creating a new SQLQuery.
        """
        if target is None:
            target = SQLQuery()

        target_items = target.items

        if prefix:
            target_items.append(prefix)

        for i, item in enumerate(items):
            if i != 0:
                target_items.append(sep)
            if isinstance(item, SQLQuery):
                target_items.extend(item.items)
            else:
                target_items.append(item)

        if suffix:
            target_items.append(suffix)
        return target

    join = staticmethod(join)

    def _str(self):
        try:
            return self.query() % tuple([sqlify(x) for x in self.values()])
        except (ValueError, TypeError):
            return self.query()

    def __str__(self):
        return safestr(self._str())

    def __unicode__(self):
        return safeunicode(self._str())

    def __repr__(self):
        return '<sql: %s>' % repr(str(self))


class SQLLiteral:
    """
    Protects a string from `sqlquote`.

        >>> sqlquote('NOW()')
        <sql: "'NOW()'">
        >>> sqlquote(SQLLiteral('NOW()'))
        <sql: 'NOW()'>
    """

    def __init__(self, v):
        self.v = v

    def __repr__(self):
        return self.v


sqlliteral = SQLLiteral


def _sqllist(values):
    """
        >>> _sqllist([1, 2, 3])
        <sql: '(1, 2, 3)'>
    """
    items = []
    items.append('(')
    for i, v in enumerate(values):
        if i != 0:
            items.append(', ')
        items.append(sqlparam(v))
    items.append(')')
    return SQLQuery(items)


def reparam(string_, dictionary):
    """
    Takes a string and a dictionary and interpolates the string
    using values from the dictionary. Returns an `SQLQuery` for the result.

        >>> reparam("s = $s", dict(s=True))
        <sql: "s = 't'">
        >>> reparam("s IN $s", dict(s=[1, 2]))
        <sql: 's IN (1, 2)'>
    """

    dictionary = dictionary.copy() # eval mucks with it
    vals = []
    result = []
    for live, chunk in _interpolate(string_):
        if live:
            v = eval(chunk, dictionary)
            result.append(sqlquote(v))
        else:
            result.append(chunk)
    return SQLQuery.join(result, '')


def sqlify(obj):
    """
    converts `obj` to its proper SQL version

        >>> sqlify(None)
        'NULL'
        >>> sqlify(True)
        "'t'"
        >>> sqlify(3)
        '3'
    """
    # because `1 == True and hash(1) == hash(True)`
    # we have to do this the hard way...

    if obj is None:
        return 'NULL'
    elif obj is True:
        return "'t'"
    elif obj is False:
        return "'f'"
    elif datetime and isinstance(obj, datetime.datetime):
        return repr(obj.isoformat())
    else:
        if isinstance(obj, unicode): obj = obj.encode('utf8')
        return repr(obj)


def sqllist(lst):
    """
    Converts the arguments for use in something like a WHERE clause.

        >>> sqllist(['a', 'b'])
        'a, b'
        >>> sqllist('a')
        'a'
        >>> sqllist(u'abc')
        u'abc'
    """
    if isinstance(lst, basestring):
        return lst
    else:
        return ', '.join(lst)


def sqlors(left, lst):
    """
    `left is a SQL clause like `tablename.arg = `
    and `lst` is a list of values. Returns a reparam-style
    pair featuring the SQL that ORs together the clause
    for each item in the lst.

        >>> sqlors('foo = ', [])
        <sql: '1=2'>
        >>> sqlors('foo = ', [1])
        <sql: 'foo = 1'>
        >>> sqlors('foo = ', 1)
        <sql: 'foo = 1'>
        >>> sqlors('foo = ', [1,2,3])
        <sql: '(foo = 1 OR foo = 2 OR foo = 3 OR 1=2)'>
    """
    if isinstance(lst, iters):
        lst = list(lst)
        ln = len(lst)
        if ln == 0:
            return SQLQuery("1=2")
        if ln == 1:
            lst = lst[0]

    if isinstance(lst, iters):
        return SQLQuery(['('] +
                        sum([[left, sqlparam(x), ' OR '] for x in lst], []) +
                        ['1=2)']
        )
    else:
        return left + sqlparam(lst)


def sqlwhere(dictionary, grouping=' AND '):
    """
    Converts a `dictionary` to an SQL WHERE clause `SQLQuery`.

        >>> sqlwhere({'cust_id': 2, 'order_id':3})
        <sql: 'order_id = 3 AND cust_id = 2'>
        >>> sqlwhere({'cust_id': 2, 'order_id':3}, grouping=', ')
        <sql: 'order_id = 3, cust_id = 2'>
        >>> sqlwhere({'a': 'a', 'b': 'b'}).query()
        'a = %s AND b = %s'
    """
    return SQLQuery.join([k + ' = ' + sqlparam(v) for k, v in dictionary.items()], grouping)


def sqlquote(a):
    """
    Ensures `a` is quoted properly for use in a SQL query.

        >>> 'WHERE x = ' + sqlquote(True) + ' AND y = ' + sqlquote(3)
        <sql: "WHERE x = 't' AND y = 3">
        >>> 'WHERE x = ' + sqlquote(True) + ' AND y IN ' + sqlquote([2, 3])
        <sql: "WHERE x = 't' AND y IN (2, 3)">
    """
    if isinstance(a, list):
        return _sqllist(a)
    else:
        return sqlparam(a).sqlquery()


class Transaction:
    """Database transaction."""

    def __init__(self, ctx):
        self.ctx = ctx
        self.transaction_count = transaction_count = len(ctx.transactions)

        class transaction_engine:
            """Transaction Engine used in top level transactions."""

            def do_transact(self):
                ctx.commit(unload=False)

            def do_commit(self):
                ctx.commit()

            def do_rollback(self):
                ctx.rollback()

        class subtransaction_engine:
            """Transaction Engine used in sub transactions."""

            def query(self, q):
                db_cursor = ctx.db.cursor()
                ctx.db_execute(db_cursor, SQLQuery(q % transaction_count))

            def do_transact(self):
                self.query('SAVEPOINT webpy_sp_%s')

            def do_commit(self):
                self.query('RELEASE SAVEPOINT webpy_sp_%s')

            def do_rollback(self):
                self.query('ROLLBACK TO SAVEPOINT webpy_sp_%s')

        class dummy_engine:
            """Transaction Engine used instead of subtransaction_engine
            when sub transactions are not supported."""
            do_transact = do_commit = do_rollback = lambda self: None

        if self.transaction_count:
            # nested transactions are not supported in some databases
            if self.ctx.get('ignore_nested_transactions'):
                self.engine = dummy_engine()
            else:
                self.engine = subtransaction_engine()
        else:
            self.engine = transaction_engine()

        self.engine.do_transact()
        self.ctx.transactions.append(self)

    def __enter__(self):
        return self

    def __exit__(self, exctype, excvalue, traceback):
        if exctype is not None:
            self.rollback()
        else:
            self.commit()

    def commit(self):
        if len(self.ctx.transactions) > self.transaction_count:
            self.engine.do_commit()
            self.ctx.transactions = self.ctx.transactions[:self.transaction_count]

    def rollback(self):
        if len(self.ctx.transactions) > self.transaction_count:
            self.engine.do_rollback()
            self.ctx.transactions = self.ctx.transactions[:self.transaction_count]


class DB:
    """Database"""

    def __init__(self, db_module, keywords):
        """Creates a database.
        """
        # some DB implementaions take optional paramater `driver` to use a specific driver modue
        # but it should not be passed to connect
        keywords.pop('driver', None)

        self.db_module = db_module
        self.keywords = keywords

        self._ctx = threadeddict()
        # flag to enable/disable printing queries
        self.printing = config.get('debug_sql', config.get('debug', False))
        self.supports_multiple_insert = False

        try:
            import DBUtils
            # enable pooling if DBUtils module is available.
            self.has_pooling = True
        except ImportError:
            self.has_pooling = False

        # Pooling can be disabled by passing pooling=False in the keywords.
        self.has_pooling = self.keywords.pop('pooling', True) and self.has_pooling

    def _getctx(self):
        if not self._ctx.get('db'):
            self._load_context(self._ctx)
        return self._ctx

    ctx = property(_getctx)

    def _load_context(self, ctx):
        ctx.dbq_count = 0
        ctx.transactions = [] # stack of transactions

        if self.has_pooling:
            ctx.db = self._connect_with_pooling(self.keywords)
        else:
            ctx.db = self._connect(self.keywords)
        ctx.db_execute = self._db_execute

        if not hasattr(ctx.db, 'commit'):
            ctx.db.commit = lambda: None

        if not hasattr(ctx.db, 'rollback'):
            ctx.db.rollback = lambda: None

        def commit(unload=True):
            # do db commit and release the connection if pooling is enabled.
            ctx.db.commit()
            if unload and self.has_pooling:
                self._unload_context(self._ctx)

        def rollback():
            # do db rollback and release the connection if pooling is enabled.
            ctx.db.rollback()
            if self.has_pooling:
                self._unload_context(self._ctx)

        ctx.commit = commit
        ctx.rollback = rollback

    def _unload_context(self, ctx):
        del ctx.db

    def _connect(self, keywords):
        return self.db_module.connect(**keywords)

    def _connect_with_pooling(self, keywords):
        def get_pooled_db():
            from DBUtils import PooledDB

            # In DBUtils 0.9.3, `dbapi` argument is renamed as `creator`
            # see Bug#122112

            if PooledDB.__version__.split('.') < '0.9.3'.split('.'):
                return PooledDB.PooledDB(dbapi=self.db_module, **keywords)
            else:
                return PooledDB.PooledDB(creator=self.db_module, **keywords)

        if getattr(self, '_pooleddb', None) is None:
            self._pooleddb = get_pooled_db()

        return self._pooleddb.connection()

    def _db_cursor(self):
        return self.ctx.db.cursor()

    def _param_marker(self):
        """Returns parameter marker based on paramstyle attribute if this database."""
        style = getattr(self, 'paramstyle', 'pyformat')

        if style == 'qmark':
            return '?'
        elif style == 'numeric':
            return ':1'
        elif style in ['format', 'pyformat']:
            return '%s'
        raise UnknownParamstyle, style

    def _db_execute(self, cur, sql_query):
        """executes an sql query"""
        self.ctx.dbq_count += 1

        try:
            a = time.time()
            query, params = self._process_query(sql_query)
            out = cur.execute(query, params)
            b = time.time()
        except:
            if self.printing:
                print >> debug, 'ERR:', str(sql_query)
            if self.ctx.transactions:
                self.ctx.transactions[-1].rollback()
            else:
                self.ctx.rollback()
            raise

        if self.printing:
            print >> debug, '%s (%s): %s' % (round(b - a, 2), self.ctx.dbq_count, str(sql_query))
        return out

    def _process_query(self, sql_query):
        """Takes the SQLQuery object and returns query string and parameters.
        """
        paramstyle = getattr(self, 'paramstyle', 'pyformat')
        query = sql_query.query(paramstyle)
        params = sql_query.values()
        return query, params

    def _where(self, where, vars):
        if isinstance(where, (int, long)):
            where = "id = " + sqlparam(where)
        #@@@ for backward-compatibility
        elif isinstance(where, (list, tuple)) and len(where) == 2:
            where = SQLQuery(where[0], where[1])
        elif isinstance(where, SQLQuery):
            pass
        else:
            where = reparam(where, vars)
        return where

    def query(self, sql_query, vars=None, processed=False, _test=False):
        """
        Execute SQL query `sql_query` using dictionary `vars` to interpolate it.
        If `processed=True`, `vars` is a `reparam`-style list to use
        instead of interpolating.

            >>> db = DB(None, {})
            >>> db.query("SELECT * FROM foo", _test=True)
            <sql: 'SELECT * FROM foo'>
            >>> db.query("SELECT * FROM foo WHERE x = $x", vars=dict(x='f'), _test=True)
            <sql: "SELECT * FROM foo WHERE x = 'f'">
            >>> db.query("SELECT * FROM foo WHERE x = " + sqlquote('f'), _test=True)
            <sql: "SELECT * FROM foo WHERE x = 'f'">
        """
        if vars is None: vars = {}

        if not processed and not isinstance(sql_query, SQLQuery):
            sql_query = reparam(sql_query, vars)

        if _test: return sql_query

        db_cursor = self._db_cursor()
        self._db_execute(db_cursor, sql_query)

        if db_cursor.description:
            names = [x[0] for x in db_cursor.description]

            def iterwrapper():
                row = db_cursor.fetchone()
                while row:
                    yield storage(dict(zip(names, row)))
                    row = db_cursor.fetchone()

            out = iterbetter(iterwrapper())
            out.__len__ = lambda: int(db_cursor.rowcount)
            out.list = lambda: [storage(dict(zip(names, x))) \
                                for x in db_cursor.fetchall()]
        else:
            out = db_cursor.rowcount

        if not self.ctx.transactions:
            self.ctx.commit()
        return out

    def select(self, tables, vars=None, what='*', where=None, order=None, group=None,
               limit=None, offset=None, _test=False):
        """
        Selects `what` from `tables` with clauses `where`, `order`,
        `group`, `limit`, and `offset`. Uses vars to interpolate.
        Otherwise, each clause can be a SQLQuery.

            >>> db = DB(None, {})
            >>> db.select('foo', _test=True)
            <sql: 'SELECT * FROM foo'>
            >>> db.select(['foo', 'bar'], where="foo.bar_id = bar.id", limit=5, _test=True)
            <sql: 'SELECT * FROM foo, bar WHERE foo.bar_id = bar.id LIMIT 5'>
        """
        if vars is None: vars = {}
        sql_clauses = self.sql_clauses(what, tables, where, group, order, limit, offset)
        clauses = [self.gen_clause(sql, val, vars) for sql, val in sql_clauses if val is not None]
        qout = SQLQuery.join(clauses)
        if _test: return qout
        return self.query(qout, processed=True)

    def where(self, table, what='*', order=None, group=None, limit=None,
              offset=None, _test=False, **kwargs):
        """
        Selects from `table` where keys are equal to values in `kwargs`.

            >>> db = DB(None, {})
            >>> db.where('foo', bar_id=3, _test=True)
            <sql: 'SELECT * FROM foo WHERE bar_id = 3'>
            >>> db.where('foo', source=2, crust='dewey', _test=True)
            <sql: "SELECT * FROM foo WHERE source = 2 AND crust = 'dewey'">
            >>> db.where('foo', _test=True)
            <sql: 'SELECT * FROM foo'>
        """
        where_clauses = []
        for k, v in kwargs.iteritems():
            where_clauses.append(k + ' = ' + sqlquote(v))

        if where_clauses:
            where = SQLQuery.join(where_clauses, " AND ")
        else:
            where = None

        return self.select(table, what=what, order=order,
                           group=group, limit=limit, offset=offset, _test=_test,
                           where=where)

    def sql_clauses(self, what, tables, where, group, order, limit, offset):
        return (
            ('SELECT', what),
            ('FROM', sqllist(tables)),
            ('WHERE', where),
            ('GROUP BY', group),
            ('ORDER BY', order),
            ('LIMIT', limit),
            ('OFFSET', offset))

    def gen_clause(self, sql, val, vars):
        if isinstance(val, (int, long)):
            if sql == 'WHERE':
                nout = 'id = ' + sqlquote(val)
            else:
                nout = SQLQuery(val)
        #@@@
        elif isinstance(val, (list, tuple)) and len(val) == 2:
            nout = SQLQuery(val[0], val[1]) # backwards-compatibility
        elif isinstance(val, SQLQuery):
            nout = val
        else:
            nout = reparam(val, vars)

        def xjoin(a, b):
            if a and b:
                return a + ' ' + b
            else:
                return a or b

        return xjoin(sql, nout)

    def insert(self, tablename, seqname=None, _test=False, **values):
        """
        Inserts `values` into `tablename`. Returns current sequence ID.
        Set `seqname` to the ID if it's not the default, or to `False`
        if there isn't one.

            >>> db = DB(None, {})
            >>> q = db.insert('foo', name='bob', age=2, created=SQLLiteral('NOW()'), _test=True)
            >>> q
            <sql: "INSERT INTO foo (age, name, created) VALUES (2, 'bob', NOW())">
            >>> q.query()
            'INSERT INTO foo (age, name, created) VALUES (%s, %s, NOW())'
            >>> q.values()
            [2, 'bob']
        """

        def q(x):
            return "(" + x + ")"

        if values:
            _keys = SQLQuery.join(values.keys(), ', ')
            _values = SQLQuery.join([sqlparam(v) for v in values.values()], ', ')
            sql_query = "INSERT INTO %s " % tablename + q(_keys) + ' VALUES ' + q(_values)
        else:
            sql_query = SQLQuery(self._get_insert_default_values_query(tablename))

        if _test: return sql_query

        db_cursor = self._db_cursor()
        if seqname is not False:
            sql_query = self._process_insert_query(sql_query, tablename, seqname)

        if isinstance(sql_query, tuple):
            # for some databases, a separate query has to be made to find
            # the id of the inserted row.
            q1, q2 = sql_query
            self._db_execute(db_cursor, q1)
            self._db_execute(db_cursor, q2)
        else:
            self._db_execute(db_cursor, sql_query)

        try:
            out = db_cursor.fetchone()[0]
        except Exception:
            out = None

        if not self.ctx.transactions:
            self.ctx.commit()
        return out

    def _get_insert_default_values_query(self, table):
        return "INSERT INTO %s DEFAULT VALUES" % table

    def multiple_insert(self, tablename, values, seqname=None, _test=False):
        """
        Inserts multiple rows into `tablename`. The `values` must be a list of dictioanries,
        one for each row to be inserted, each with the same set of keys.
        Returns the list of ids of the inserted rows.
        Set `seqname` to the ID if it's not the default, or to `False`
        if there isn't one.

            >>> db = DB(None, {})
            >>> db.supports_multiple_insert = True
            >>> values = [{"name": "foo", "email": "foo@example.com"}, {"name": "bar", "email": "bar@example.com"}]
            >>> db.multiple_insert('person', values=values, _test=True)
            <sql: "INSERT INTO person (name, email) VALUES ('foo', 'foo@example.com'), ('bar', 'bar@example.com')">
        """
        if not values:
            return []

        if not self.supports_multiple_insert:
            out = [self.insert(tablename, seqname=seqname, _test=_test, **v) for v in values]
            if seqname is False:
                return None
            else:
                return out

        keys = values[0].keys()
        #@@ make sure all keys are valid

        for v in values:
            if v.keys() != keys:
                raise ValueError, 'Not all rows have the same keys'

        sql_query = SQLQuery('INSERT INTO %s (%s) VALUES ' % (tablename, ', '.join(keys)))

        for i, row in enumerate(values):
            if i != 0:
                sql_query.append(", ")
            SQLQuery.join([SQLParam(row[k]) for k in keys], sep=", ", target=sql_query, prefix="(", suffix=")")

        if _test: return sql_query

        db_cursor = self._db_cursor()
        if seqname is not False:
            sql_query = self._process_insert_query(sql_query, tablename, seqname)

        if isinstance(sql_query, tuple):
            # for some databases, a separate query has to be made to find
            # the id of the inserted row.
            q1, q2 = sql_query
            self._db_execute(db_cursor, q1)
            self._db_execute(db_cursor, q2)
        else:
            self._db_execute(db_cursor, sql_query)

        try:
            out = db_cursor.fetchone()[0]
            out = range(out - len(values) + 1, out + 1)
        except Exception:
            out = None

        if not self.ctx.transactions:
            self.ctx.commit()
        return out


    def update(self, tables, where, vars=None, _test=False, **values):
        """
        Update `tables` with clause `where` (interpolated using `vars`)
        and setting `values`.

            >>> db = DB(None, {})
            >>> name = 'Joseph'
            >>> q = db.update('foo', where='name = $name', name='bob', age=2,
            ...     created=SQLLiteral('NOW()'), vars=locals(), _test=True)
            >>> q
            <sql: "UPDATE foo SET age = 2, name = 'bob', created = NOW() WHERE name = 'Joseph'">
            >>> q.query()
            'UPDATE foo SET age = %s, name = %s, created = NOW() WHERE name = %s'
            >>> q.values()
            [2, 'bob', 'Joseph']
        """
        if vars is None: vars = {}
        where = self._where(where, vars)

        query = (
            "UPDATE " + sqllist(tables) +
            " SET " + sqlwhere(values, ', ') +
            " WHERE " + where)

        if _test: return query

        db_cursor = self._db_cursor()
        self._db_execute(db_cursor, query)
        if not self.ctx.transactions:
            self.ctx.commit()
        return db_cursor.rowcount

    def delete(self, table, where, using=None, vars=None, _test=False):
        """
        Deletes from `table` with clauses `where` and `using`.

            >>> db = DB(None, {})
            >>> name = 'Joe'
            >>> db.delete('foo', where='name = $name', vars=locals(), _test=True)
            <sql: "DELETE FROM foo WHERE name = 'Joe'">
        """
        if vars is None: vars = {}
        where = self._where(where, vars)

        q = 'DELETE FROM ' + table
        if using: q += ' USING ' + sqllist(using)
        if where: q += ' WHERE ' + where

        if _test: return q

        db_cursor = self._db_cursor()
        self._db_execute(db_cursor, q)
        if not self.ctx.transactions:
            self.ctx.commit()
        return db_cursor.rowcount

    def _process_insert_query(self, query, tablename, seqname):
        return query

    def transaction(self):
        """Start a transaction."""
        return Transaction(self.ctx)


class PostgresDB(DB):
    """Postgres driver."""

    def __init__(self, **keywords):
        if 'pw' in keywords:
            keywords['password'] = keywords.pop('pw')

        db_module = import_driver(["psycopg2", "psycopg", "pgdb"], preferred=keywords.pop('driver', None))
        if db_module.__name__ == "psycopg2":
            import psycopg2.extensions

            psycopg2.extensions.register_type(psycopg2.extensions.UNICODE)
        if db_module.__name__ == "pgdb" and 'port' in keywords:
            keywords["host"] += ":" + str(keywords.pop('port'))

        # if db is not provided postgres driver will take it from PGDATABASE environment variable
        if 'db' in keywords:
            keywords['database'] = keywords.pop('db')

        self.dbname = "postgres"
        self.paramstyle = db_module.paramstyle
        DB.__init__(self, db_module, keywords)
        self.supports_multiple_insert = True
        self._sequences = None

    def _process_insert_query(self, query, tablename, seqname):
        if seqname is None:
            # when seqname is not provided guess the seqname and make sure it exists
            seqname = tablename + "_id_seq"
            if seqname not in self._get_all_sequences():
                seqname = None

        if seqname:
            query += "; SELECT currval('%s')" % seqname

        return query

    def _get_all_sequences(self):
        """Query postgres to find names of all sequences used in this database."""
        if self._sequences is None:
            q = "SELECT c.relname FROM pg_class c WHERE c.relkind = 'S'"
            self._sequences = set([c.relname for c in self.query(q)])
        return self._sequences

    def _connect(self, keywords):
        conn = DB._connect(self, keywords)
        try:
            conn.set_client_encoding('UTF8')
        except AttributeError:
            # fallback for pgdb driver
            conn.cursor().execute("set client_encoding to 'UTF-8'")
        return conn

    def _connect_with_pooling(self, keywords):
        conn = DB._connect_with_pooling(self, keywords)
        conn._con._con.set_client_encoding('UTF8')
        return conn


class MySQLDB(DB):
    def __init__(self, **keywords):
        import MySQLdb as db

        if 'pw' in keywords:
            keywords['passwd'] = keywords['pw']
            del keywords['pw']

        if 'charset' not in keywords:
            keywords['charset'] = 'utf8'
        elif keywords['charset'] is None:
            del keywords['charset']

        self.paramstyle = db.paramstyle = 'pyformat' # it's both, like psycopg
        self.dbname = "mysql"
        DB.__init__(self, db, keywords)
        self.supports_multiple_insert = True

    def _process_insert_query(self, query, tablename, seqname):
        return query, SQLQuery('SELECT last_insert_id();')

    def _get_insert_default_values_query(self, table):
        return "INSERT INTO %s () VALUES()" % table


def import_driver(drivers, preferred=None):
    """Import the first available driver or preferred driver.
    """
    if preferred:
        drivers = [preferred]

    for d in drivers:
        try:
            return __import__(d, None, None, ['x'])
        except ImportError:
            pass
    raise ImportError("Unable to import " + " or ".join(drivers))


class SqliteDB(DB):
    def __init__(self, **keywords):
        db = import_driver(["sqlite3", "pysqlite2.dbapi2", "sqlite"], preferred=keywords.pop('driver', None))

        if db.__name__ in ["sqlite3", "pysqlite2.dbapi2"]:
            db.paramstyle = 'qmark'

        # sqlite driver doesn't create datatime objects for timestamp columns unless `detect_types` option is passed.
        # It seems to be supported in sqlite3 and pysqlite2 drivers, not surte about sqlite.
        keywords.setdefault('detect_types', db.PARSE_DECLTYPES)

        self.paramstyle = db.paramstyle
        keywords['database'] = keywords.pop('db')
        keywords['pooling'] = False # sqlite don't allows connections to be shared by threads
        self.dbname = "sqlite"
        DB.__init__(self, db, keywords)

    def _process_insert_query(self, query, tablename, seqname):
        return query, SQLQuery('SELECT last_insert_rowid();')

    def query(self, *a, **kw):
        out = DB.query(self, *a, **kw)
        if isinstance(out, iterbetter):
            del out.__len__
        return out


class FirebirdDB(DB):
    """Firebird Database.
    """

    def __init__(self, **keywords):
        try:
            import kinterbasdb as db
        except Exception:
            db = None
            pass
        if 'pw' in keywords:
            keywords['password'] = keywords.pop('pw')
        keywords['database'] = keywords.pop('db')

        self.paramstyle = db.paramstyle

        DB.__init__(self, db, keywords)

    def delete(self, table, where=None, using=None, vars=None, _test=False):
        # firebird doesn't support using clause
        using = None
        return DB.delete(self, table, where, using, vars, _test)

    def sql_clauses(self, what, tables, where, group, order, limit, offset):
        return (
            ('SELECT', ''),
            ('FIRST', limit),
            ('SKIP', offset),
            ('', what),
            ('FROM', sqllist(tables)),
            ('WHERE', where),
            ('GROUP BY', group),
            ('ORDER BY', order)
        )


class MSSQLDB(DB):
    def __init__(self, **keywords):
        import pymssql as db

        if 'pw' in keywords:
            keywords['password'] = keywords.pop('pw')
        keywords['database'] = keywords.pop('db')
        self.dbname = "mssql"
        DB.__init__(self, db, keywords)

    def _process_query(self, sql_query):
        """Takes the SQLQuery object and returns query string and parameters.
        """
        # MSSQLDB expects params to be a tuple.
        # Overwriting the default implementation to convert params to tuple.
        paramstyle = getattr(self, 'paramstyle', 'pyformat')
        query = sql_query.query(paramstyle)
        params = sql_query.values()
        return query, tuple(params)

    def sql_clauses(self, what, tables, where, group, order, limit, offset):
        return (
            ('SELECT', what),
            ('TOP', limit),
            ('FROM', sqllist(tables)),
            ('WHERE', where),
            ('GROUP BY', group),
            ('ORDER BY', order),
            ('OFFSET', offset))

    def _test(self):
        """Test LIMIT.

            Fake presence of pymssql module for running tests.
            >>> import sys
            >>> sys.modules['pymssql'] = sys.modules['sys']

            MSSQL has TOP clause instead of LIMIT clause.
            >>> db = MSSQLDB(db='test', user='joe', pw='secret')
            >>> db.select('foo', limit=4, _test=True)
            <sql: 'SELECT * TOP 4 FROM foo'>
        """
        pass


class OracleDB(DB):
    def __init__(self, **keywords):
        import cx_Oracle as db

        if 'pw' in keywords:
            keywords['password'] = keywords.pop('pw')

            #@@ TODO: use db.makedsn if host, port is specified
        keywords['dsn'] = keywords.pop('db')
        self.dbname = 'oracle'
        db.paramstyle = 'numeric'
        self.paramstyle = db.paramstyle

        # oracle doesn't support pooling
        keywords.pop('pooling', None)
        DB.__init__(self, db, keywords)

    def _process_insert_query(self, query, tablename, seqname):
        if seqname is None:
        # It is not possible to get seq name from table name in Oracle
            return query
        else:
            return query + "; SELECT %s.currval FROM dual" % seqname


def dburl2dict(url):
    """
    Takes a URL to a database and parses it into an equivalent dictionary.

        >>> dburl2dict('postgres://james:day@serverfarm.example.net:5432/mygreatdb')
        {'pw': 'day', 'dbn': 'postgres', 'db': 'mygreatdb', 'host': 'serverfarm.example.net', 'user': 'james', 'port': '5432'}
        >>> dburl2dict('postgres://james:day@serverfarm.example.net/mygreatdb')
        {'user': 'james', 'host': 'serverfarm.example.net', 'db': 'mygreatdb', 'pw': 'day', 'dbn': 'postgres'}
        >>> dburl2dict('postgres://james:d%40y@serverfarm.example.net/mygreatdb')
        {'user': 'james', 'host': 'serverfarm.example.net', 'db': 'mygreatdb', 'pw': 'd@y', 'dbn': 'postgres'}
    """
    dbn, rest = url.split('://', 1)
    user, rest = rest.split(':', 1)
    pw, rest = rest.split('@', 1)
    if ':' in rest:
        host, rest = rest.split(':', 1)
        port, rest = rest.split('/', 1)
    else:
        host, rest = rest.split('/', 1)
        port = None
    db = rest

    uq = urllib.unquote
    out = dict(dbn=dbn, user=uq(user), pw=uq(pw), host=uq(host), db=uq(db))
    if port: out['port'] = port
    return out


_databases = {}


def database(dburl=None, **params):
    """Creates appropriate database using params.

    Pooling will be enabled if DBUtils module is available.
    Pooling can be disabled by passing pooling=False in params.
    """
    if not dburl and not params:
        dburl = os.environ['DATABASE_URL']
    if dburl:
        params = dburl2dict(dburl)
    dbn = params.pop('dbn')
    if dbn in _databases:
        return _databases[dbn](**params)
    else:
        raise UnknownDB, dbn


def register_database(name, clazz):
    """
    Register a database.

        >>> class LegacyDB(DB):
        ...     def __init__(self, **params):
        ...        pass
        ...
        >>> register_database('legacy', LegacyDB)
        >>> db = database(dbn='legacy', db='test', user='joe', passwd='secret')
    """
    _databases[name] = clazz


register_database('mysql', MySQLDB)
register_database('postgres', PostgresDB)
register_database('sqlite', SqliteDB)
register_database('firebird', FirebirdDB)
register_database('mssql', MSSQLDB)
register_database('oracle', OracleDB)


def _interpolate(format):
    """
    Takes a format string and returns a list of 2-tuples of the form
    (boolean, string) where boolean says whether string should be evaled
    or not.

    from <http://lfw.org/python/Itpl.py> (public domain, Ka-Ping Yee)
    """
    from tokenize import tokenprog

    def matchorfail(text, pos):
        match = tokenprog.match(text, pos)
        if match is None:
            raise _ItplError(text, pos)
        return match, match.end()

    namechars = "abcdefghijklmnopqrstuvwxyz" \
                "ABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789_";
    chunks = []
    pos = 0

    while 1:
        dollar = format.find("$", pos)
        if dollar < 0:
            break
        nextchar = format[dollar + 1]

        if nextchar == "{":
            chunks.append((0, format[pos:dollar]))
            pos, level = dollar + 2, 1
            while level:
                match, pos = matchorfail(format, pos)
                tstart, tend = match.regs[3]
                token = format[tstart:tend]
                if token == "{":
                    level = level + 1
                elif token == "}":
                    level = level - 1
            chunks.append((1, format[dollar + 2:pos - 1]))

        elif nextchar in namechars:
            chunks.append((0, format[pos:dollar]))
            match, pos = matchorfail(format, dollar + 1)
            while pos < len(format):
                if format[pos] == "." and \
                                        pos + 1 < len(format) and format[pos + 1] in namechars:
                    match, pos = matchorfail(format, pos + 1)
                elif format[pos] in "([":
                    pos, level = pos + 1, 1
                    while level:
                        match, pos = matchorfail(format, pos)
                        tstart, tend = match.regs[3]
                        token = format[tstart:tend]
                        if token[0] in "([":
                            level = level + 1
                        elif token[0] in ")]":
                            level = level - 1
                else:
                    break
            chunks.append((1, format[dollar + 1:pos]))
        else:
            chunks.append((0, format[pos:dollar + 1]))
            pos = dollar + 1 + (nextchar == "$")

    if pos < len(format):
        chunks.append((0, format[pos:]))
    return chunks


if __name__ == "__main__":
    import doctest

    doctest.testmod()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
access log中间件，替换tornado的log_request实现插件式日志输出
"""
from datetime import datetime
import logging

access_log = logging.getLogger('torngas.accesslog')


def log_request(handler):
    _datetime = datetime.now().strftime('%Y-%m-%d %H:%M:%S')
    request_time = 1000.0 * handler.request.request_time()
    user_agent = handler.request.headers.get("User-Agent", "-")
    remote_ip = handler.request.remote_ip
    uri = handler.request.uri
    method = handler.request.method
    version = handler.request.version
    status = handler.get_status()
    referer = handler.request.headers.get('Referer', "-")
    content_length = handler._headers.get("Content-Length", "-")
    _message = '%s - - [%s] "%s %s %s" %s %s "%s" "%s" %dms' % (
        remote_ip,
        _datetime,
        method,
        uri,
        version,
        status,
        content_length,
        referer,
        user_agent,
        request_time
    )
    access_log.info(_message)


class AccessLogMiddleware(object):
    def process_init(self, application):
        def _(__):
            pass

        application.settings['log_function'] = _

    def process_endcall(self, handler, clear):
        log_request(handler)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Created by mengqingyun on 14-5-21.
"""

from tornado.web import HTTPError

try:
    from exceptions import Exception, StandardError, Warning
except ImportError:
    # Python 3
    StandardError = Exception


class BaseError(StandardError):
    """Base Error"""


class ArgumentError(BaseError):
    """Arguments error"""


class ConfigError(BaseError):
    """raise config error"""


class UrlError(BaseError):
    """route write error"""


class Http404(HTTPError):
    def __init__(self, log_message='not found', *args, **kwargs):
        super(Http404, self).__init__(404, log_message, *args, **kwargs)


class HttpForbiddenError(HTTPError):
    def __init__(self, log_message='forbidden', *args, **kwargs):
        super(HttpForbiddenError, self).__init__(403, log_message, *args, **kwargs)


class HttpNotAllowError(HTTPError):
    def __init__(self, log_message='method not allowed', *args, **kwargs):
        super(HttpNotAllowError, self).__init__(405, log_message, *args, **kwargs)


class HttpBadRequestError(HTTPError):
    def __init__(self, log_message='bad request', *args, **kwargs):
        super(HttpBadRequestError, self).__init__(400, log_message, *args, **kwargs)


class Http500(HTTPError):
    def __init__(self, log_message='server error', *args, **kwargs):
        super(Http500, self).__init__(500, log_message, *args, **kwargs)




#!/usr/bin/env python
# -*- coding: utf-8 -*-
import traceback
from tornado import web
from tornado import version_info
from logger import SysLogger
from middleware.manager import Manager
from tornado import httputil


class Application(web.Application):
    def __init__(self, handlers=None,
                 default_host="",
                 transforms=None,
                 wsgi=False,
                 middlewares=None,
                 **settings):

        super(Application, self).__init__(
            handlers=handlers,
            default_host=default_host,
            transforms=transforms,
            wsgi=wsgi, **settings)

        self.middleware_fac = Manager()
        if middlewares:
            self.middleware_fac.register_all(middlewares)
            self.middleware_fac.run_init(self)

        if version_info[0] > 3:
            this = self

            class HttpRequest(httputil.HTTPServerRequest):
                def __init__(self, *args, **kwargs):
                    super(HttpRequest, self).__init__(*args, **kwargs)
                    this.middleware_fac.set_request(self)
                    try:
                        result = this.middleware_fac.run_call(self)
                        this.middleware_fac.catch_middleware_exc(result)
                    except Exception:
                        SysLogger.trace_logger.error(traceback.format_exc())

            httputil.HTTPServerRequest = HttpRequest

    def __call__(self, request):
        if version_info[0] < 4:
            try:
                self.middleware_fac.set_request(request)
                self.middleware_fac.run_call(request)
                return web.Application.__call__(self, request)

            except Exception, e:
                SysLogger.trace_logger.error(e)
                raise



"""
Caching framework.
"""
from threading import local
from torngas.settings_manager import settings

from torngas.cache.backends.base import (
    InvalidCacheBackendError, CacheKeyWarning, BaseCache)

from tornado.util import import_object
from torngas.exception import ConfigError
from tornado.ioloop import PeriodicCallback


BACKENDS = {
    'memcached': 'memcached',
    'localcache': 'localcache',
    'dummy': 'dummy',
    'redis': 'rediscache'
}

DEFAULT_CACHE_ALIAS = 'default'
DEFAULT_REDIS_ALIAS = 'default_redis'
DEFAULT_MEMCACHED_ALIAS = 'default_memcache'
DEFAULT_DUMMY_ALIAS = 'dummy'

if DEFAULT_CACHE_ALIAS not in settings.CACHES:
    raise ConfigError("You must define a '%s' cache" % DEFAULT_CACHE_ALIAS)


def _create_cache(backend, **kwargs):
    try:
        # Try to get the CACHES entry for the given backend name first
        try:
            conf = settings.CACHES[backend]
        except KeyError:
            try:
                # Trying to import the given backend, in case it's a dotted path
                import_object(backend)
            except ImportError as e:
                raise InvalidCacheBackendError("Could not find backend '%s': %s" % (
                    backend, e))
            location = kwargs.pop('LOCATION', '')
            params = kwargs
        else:
            params = conf.copy()
            params.update(kwargs)
            backend = params.pop('BACKEND')
            location = params.pop('LOCATION', '')
        backend_cls = import_object(backend)
    except ImportError as e:
        raise InvalidCacheBackendError(
            "Could not find backend '%s': %s" % (backend, e))
    return backend_cls(location, params)


class CacheHandler(object):
    """
    A Cache Handler to manage access to Cache instances.

    Ensures only one instance of each alias exists per thread.
    """

    def __init__(self):
        self._caches = local()

    def __getitem__(self, alias):
        try:
            return self._caches.caches[alias]
        except AttributeError:
            self._caches.caches = {}
        except KeyError:
            pass

        if alias not in settings.CACHES:
            raise InvalidCacheBackendError(
                "Could not find config for '%s' in settings.CACHES" % alias
            )

        cache = _create_cache(alias)
        if hasattr(cache, 'clear_expires'):
            PeriodicCallback(cache.clear_expires, 1000 * 1800).start()
        self._caches.caches[alias] = cache
        return cache


    def all(self):
        return getattr(self._caches, 'caches', {}).values()


caches = CacheHandler()


class DefaultCacheProxy(object):
    """
    Proxy access to the default Cache object's attributes.

    This allows the legacy `cache` object to be thread-safe using the new
    ``caches`` API.
    """

    def __getattr__(self, name):
        return getattr(caches[DEFAULT_CACHE_ALIAS], name)

    def __setattr__(self, name, value):
        return setattr(caches[DEFAULT_CACHE_ALIAS], name, value)

    def __delattr__(self, name):
        return delattr(caches[DEFAULT_CACHE_ALIAS], name)

    def __contains__(self, key):
        return key in caches[DEFAULT_CACHE_ALIAS]

    def __eq__(self, other):
        return caches[DEFAULT_CACHE_ALIAS] == other

    def __ne__(self, other):
        return caches[DEFAULT_CACHE_ALIAS] != other


cache = DefaultCacheProxy()


def close_caches(**kwargs):
    # Some caches -- python-memcached in particular -- need to do a cleanup at the
    # end of a request cycle. If not implemented in a particular backend
    # cache.close is a no-op
    for cache in caches.all():
        cache.close()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
session中间件,支持disk，进程内缓存cache,memcache
sessioin过期策略分为三种情形：
1.固定时间过期，例如10天内没有访问则过期，timeout=xxx
    *cookie策略：每次访问时设置cookie为过期时间
    *缓存策略：每次访问设置缓存失效时间为固定过期时间
2.会话过期，关闭浏览器即过期timeout=0
    *cookie策略：岁浏览器关闭而过期
    *缓存策略：设置缓存失效期为1天，每次访问更新失效期，如果浏览器关闭，则一天后被清除

3.永不过期(记住我)
    *cookie策略：timeout1年
    *缓存策略：1年
"""
import os
import time
import hmac
import re

try:
    import cPickle as pickle
except ImportError:
    import pickle
try:
    import hashlib

    sha1 = hashlib.sha1
except ImportError:
    import sha

    sha1 = sha.new

from torngas.storage import storage
from torngas.utils import safestr
from torngas.utils import lazyimport
from torngas.settings_manager import settings
from torngas.cache import caches

cache_module = lazyimport('torngas.cache')
rx = re.compile('^[0-9a-fA-F]+$')


class SessionMiddleware(object):
    _cachestore = None
    session = None

    def process_init(self, application):
        self._cachestore = caches[settings.SESSION.session_cache_alias]

    def process_request(self, handler, clear):
        session = SessionManager(handler, self._cachestore, settings.SESSION)
        session.load_session()
        handler.session = session


    def process_response(self, handler, clear, chunk):
        if hasattr(handler, "session"):
            handler.session.save()
            del handler.session


_DAY1 = 24 * 60 * 60
_DAY30 = _DAY1 * 30
_VERIFICATION_KEY = '__VERIFSSID'

session_parameters = storage({
    'session_name': '__TORNADOSSID',
    'cookie_domain': None,
    'cookie_path': '/',
    'expires': 0,  # 24 * 60 * 60, # 24 hours in seconds
    'ignore_change_ip': False,
    'httponly': True,
    'secure': False,
    'secret_key': 'fLjUfxqXtfNoIldA0A0J',
    'session_version': ''
})


class SessionManager(object):
    _killed = False

    def __init__(self, handler, store, config=None):
        self._get_cookie = handler.get_cookie
        self._set_cookie = handler.set_cookie
        self.remote_ip = handler.request.remote_ip
        self.store = store
        self.config = storage(session_parameters)
        if config:
            self.config.update(config)

        self._data = {}

    def __contains__(self, key):
        return key in self._data

    def __setitem__(self, key, value):
        self._data[key] = value

    def __getitem__(self, key):
        return self._data.get(key, None)

    def __delitem__(self, key):
        del self._data[key]

    def get(self, key, default=None):
        return self._data.get(key, default)

    def load_session(self):
        """
        加载session
        :return:
        """
        self.sessionid = self._get_cookie(self.config.session_name)

        if self.sessionid and not self._valid_session_id(self.sessionid):
            self.expired()

        if self.sessionid:
            if self.sessionid in self.store:
                expires, _data = self.store.get(self.sessionid)
                self._data.update(_data)
                self.config.expires = expires
            self._validate_ip()
            hmac_verif = self._get_cookie(_VERIFICATION_KEY)
            if self.sessionid and hmac_verif != self._generate_hmac(self.sessionid):
                self.expired()

        if not self.sessionid:
            self.sessionid = self._create_sessionid()

        self._data['remote_ip'] = self.remote_ip

    def save(self):
        if not self._killed:
            httponly = self.config.httponly
            secure = self.config.secure
            expires = self.config.expires  # 单位:秒
            cache_expires = expires
            if expires == 0:
                # 过期时间为0时，对于tornado来说，是会话有效期，关闭浏览器失效，但是
                # 对于cache缓存而言，无法及时捕获会话结束状态，鉴于此，将cache的缓存设置为一天
                # cache在每次请求后会清理过期的缓存
                cache_expires = _DAY1

            if not secure:
                secure = ''

            if not httponly:
                httponly = ''
            set_expire = 0 if expires == 0 else time.time() + expires
            self._set_cookie(
                self.config.session_name,
                self.sessionid,
                domain=self.config.cookie_domain or '',
                expires=set_expire,
                path=self.config.cookie_path,
                secure=secure,
                httponly=httponly)
            self._set_cookie(_VERIFICATION_KEY, self._generate_hmac(self.sessionid),
                             domain=self.config.cookie_domain or '',
                             expires=set_expire,
                             path=self.config.cookie_path,
                             secure=secure,
                             httponly=httponly)
            self.store.set(self.sessionid, (expires, self._data), cache_expires)

        else:
            self._set_cookie(self.config.session_name, self.sessionid, expires=-1)
            self._set_cookie(_VERIFICATION_KEY, self._generate_hmac(self.sessionid), expires=-1)
            self.store.delete(self.sessionid)
            self.sessionid = None
            self._killed = False

    def _valid_session_id(self, session_id):
        """
        验证sessionid格式
        :return:bool
        """
        if session_id:
            sess = session_id.split('|')
            if len(sess) > 1:
                return rx.match(sess[0]) and sess[1] == self.config.session_version

    def expired(self):
        """
        强制过期
        :return:None
        """
        self._killed = True
        self.save()

    def _create_sessionid(self):
        rand = os.urandom(16)
        now = time.time()
        secret_key = self.config.secret_key
        session_id = sha1("%s%s%s%s" % (rand, now, safestr(self.remote_ip), secret_key))
        session_id = session_id.hexdigest()
        return str(session_id).upper() + '|' + self.config.session_version

    def _generate_hmac(self, session_id):
        return hmac.new(session_id, self.config.secret_key, hashlib.sha1).hexdigest()

    def _validate_ip(self):
        if self.sessionid and self._data.get('remote_ip', None) != self.remote_ip:
            if not self.config.ignore_change_ip:
                return self.expired()

    def set_expire(self, expires):
        self.config.expires = expires
        self.save()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os

DEBUG = True

XHEADERS = True

# tornado全局配置
TORNADO_CONF = {
    "static_path": "static",
    "xsrf_cookies": True,
    "login_url": '/login',
    "cookie_secret": "bXZ/gDAbQA+zaTxdqJwxKa8OZTbuZE/ok3doaow9N4Q=",
    "template_path": os.path.join(os.path.dirname(os.path.dirname(__file__)), 'templates'),
    # 安全起见，可以定期生成新的cookie 秘钥，生成方法：
    # base64.b64encode(uuid.uuid4().bytes + uuid.uuid4().bytes)
}

# ###########
# 中间件     #
# ###########
MIDDLEWARE_CLASSES = (
    'torngas.middleware.accesslog.AccessLogMiddleware',
    'torngas.middleware.session.SessionMiddleware',
    'torngas.middleware.signal.SignalMiddleware',
    'torngas.httpmodule.httpmodule.HttpModuleMiddleware',
)
INSTALLED_APPS = (
)
# 全局modules配置
COMMON_MODULES = (
    # 'module限定名',
)
# 路由modules，针对某个路由或某些路由起作用
ROUTE_MODULES = {
    # {'路由名称或正则':['module限定名','!被排除的module限定名']}
}
###########
# 缓存配置 #
###########
CACHES = {
    'default': {
        'BACKEND': 'torngas.cache.backends.localcache.LocMemCache',
        'LOCATION': 'process_cache',
        'OPTIONS': {
            'MAX_ENTRIES': 10000,
            'CULL_FREQUENCY': 3
        }
    },
    'default_memcache': {
        'BACKEND': 'torngas.cache.backends.memcached.MemcachedCache',
        'LOCATION': [
            '127.0.0.1:11211'
        ],
        'TIMEOUT': 300
    },
    'dummy': {
        'BACKEND': 'torngas.cache.backends.dummy.DummyCache'
    },
    'default_redis': {
        'BACKEND': 'torngas.cache.backends.rediscache.RedisCache',
        'LOCATION': '127.0.0.1:6379',
        'OPTIONS': {
            'DB': 0,
            'PARSER_CLASS': 'redis.connection.DefaultParser',
            'POOL_KWARGS': {
                'socket_timeout': 2,
                'socket_connect_timeout': 2
            },
            'PING_INTERVAL': 120  # 定时ping redis连接池，防止被服务端断开连接（s秒）
        }
    },

}


#################
# 本地化翻译文件地址#
#################

TRANSLATIONS = False  # 是否开启国际化
TRANSLATIONS_CONF = {
    'translations_dir': os.path.join(os.path.dirname(__file__), 'translations'),
    'locale_default': 'zh_CN',
    'use_accept_language': True
}


# 白名单未开启，如需使用，请用元祖列出白名单ip
WHITELIST = False
#######
# WHITELIST = (
#     '127.0.0.1',
# '127.0.0.2',
# )

#tornado日志功能配置
LOGGER_CONFIG = {
    "use_tornadolog": False,
    "root_level": 'INFO',
    # 日志根目录（如果某具体日志filename指定路径，则自动忽略此根目录配置）
    "root_dir": 'logs/'
}

LOGGER = {
    'tornado': {
        "OPEN": True,
        "LEVEL": "INFO",
        "HANDLERS": [
            {
                "module": "torngas.logger.UsePortRotatingFileHandler",
                "filename": "tornado.log",
                "when": "midnight",
                "encoding": "utf-8",
                "delay": True,
                "backupCount": 10,
            }
        ]
    },
    'torngas.tracelog': {
        "OPEN": True,
        "LEVEL": "ERROR",
        "HANDLERS": [
            {
                "module": "torngas.logger.UsePortRotatingFileHandler",
                "filename": "torngas_trace_log.log",
                "when": "midnight",
                "encoding": "utf-8",
                "delay": True,
                "backupCount": 20,
            }
        ]
    },
    'torngas.accesslog': {
        "OPEN": True,
        "LEVEL": "INFO",
        "FORMATTER": '%(message)s',
        "HANDLERS": [
            {
                "module": "torngas.logger.UsePortRotatingFileHandler",
                "filename": "torngas_access_log.log",
                "when": "midnight",
                "encoding": "utf-8",
                "delay": False,
                "backupCount": 20,
            }
        ]
    },
    'torngas.infolog': {
        "OPEN": True,
        "LEVEL": "INFO",
        "HANDLERS": [
            {
                "module": "torngas.logger.UsePortRotatingFileHandler",
                "filename": "torngas_info_log.log",
                "when": "midnight",
                "encoding": "utf-8",
                "delay": True,
                "backupCount": 10,
            }
        ]
    },
}

IPV4_ONLY = True


#开启session支持
SESSION = {
    'session_cache_alias': 'default',  # 'session_loccache',对应cache配置
    'session_name': '__TORNADOID',
    'cookie_domain': '',
    'cookie_path': '/',
    'expires': 0,  # 24 * 60 * 60, # 24 hours in seconds,0代表浏览器会话过期
    'ignore_change_ip': False,
    'httponly': True,
    'secure': False,
    'secret_key': 'fLjUfxqXtfNoIldA0A0J',
    'session_version': 'EtdHjDO1'
}


#配置模版引擎
#引入相应的TemplateLoader即可
#若使用自带的请给予None
#支持mako和jinja2
#mako设置为torngas.template.mako_loader.MakoTemplateLoader
#jinj2设置为torngas.template.jinja2_loader.Jinja2TemplateLoader
#初始化参数请参照jinja的Environment或mako的TemplateLookup,不再详细给出
TEMPLATE_CONFIG = {
    'template_engine': None,
    #模版路径由torngas.handler中commonhandler重写，无需指定，模版将存在于每个应用的根目录下
    'filesystem_checks': True,  #通用选项
    'cache_directory': '../_tmpl_cache',  #模版编译文件目录,通用选项
    'collection_size': 50,  #暂存入内存的模版项，可以提高性能，mako选项,详情见mako文档
    'cache_size': 0,  #类似于mako的collection_size，设定为-1为不清理缓存，0则每次都会重编译模板
    'format_exceptions': False,  #格式化异常输出，mako专用
    'autoescape': False  #默认转义设定，jinja2专用

}


# 数据库连接字符串，
# 元祖，每组为n个数据库连接，有且只有一个master，可配与不配slave
DATABASE_CONNECTION = {
    'default': {
        'connections': [{
                            'ROLE': 'master',
                            'DRIVER': 'mysql+mysqldb',
                            'UID': 'root',
                            'PASSWD': '',
                            'HOST': '',
                            'PORT': 3306,
                            'DATABASE': '',
                            'QUERY': {"charset": "utf8"}

                        },
                        {
                            'ROLE': 'slave',
                            'DRIVER': 'mysql+mysqldb',
                            'UID': 'root',
                            'PASSWD': '',
                            'HOST': '',
                            'PORT': 3306,
                            'DATABASE': '',
                            'QUERY': {"charset": "utf8"}

                        }]
    }
}
# 每个定时对db进行一次ping操作，防止mysql gone away
PING_DB = 300  # (s秒)
# sqlalchemy配置，列出部分，可自行根据sqlalchemy文档增加配置项
# 该配置项对所有连接全局共享
SQLALCHEMY_CONFIGURATION = {
    'sqlalchemy.connect_args': {
        'connect_timeout': 3
    },
    'sqlalchemy.echo': False,
    'sqlalchemy.max_overflow': 10,
    'sqlalchemy.echo_pool': False,
    'sqlalchemy.pool_timeout': 5,
    'sqlalchemy.encoding': 'utf-8',
    'sqlalchemy.pool_size': 100,
    'sqlalchemy.pool_recycle': 3600,
    'sqlalchemy.poolclass': 'QueuePool'  # 手动指定连接池类
}


#!/usr/bin/env python
# -*- coding: utf-8 -*-
import sys
from base import CacheMixin, CacheClient, InvalidCacheBackendError, DEFAULT_TIMEOUT
from torngas.storage import SortedDict
from torngas.utils import safestr
from torngas.exception import ConfigError
from tornado.util import import_object

try:
    import cPickle as pickle
except ImportError:
    import pickle

try:
    import redis
except ImportError:
    raise InvalidCacheBackendError(
        "Redis cache backend requires the 'redis-py' library")
from redis.connection import UnixDomainSocketConnection, Connection
from redis.connection import DefaultParser

PY3 = (sys.version_info >= (3,))

if PY3:
    bytes_type = bytes
else:
    bytes_type = str


def python_2_unicode_compatible(klass):
    """
    A decorator that defines __unicode__ and __str__ methods under Python 2.
    Under Python 3 it does nothing.

    To support Python 2 and 3 with a single code base, define a __str__ method
    returning text and apply this decorator to the class.
    """
    if not PY3:
        klass.__unicode__ = klass.__str__
        klass.__str__ = lambda self: self.__unicode__().encode('utf-8')
    return klass


@python_2_unicode_compatible
class CacheKey(object):
    """
    A stub string class that we can use to check if a key was created already.
    """

    def __init__(self, key):
        self._key = key

    def __eq__(self, other):
        return self._key == other

    def __str__(self):
        return safestr(self._key)

    def __repr__(self):
        return repr(self._key)

    def __hash__(self):
        return hash(self._key)


class CacheConnectionPool(object):
    def __init__(self):
        self._connection_pools = {}

    def get_connection_pool(self, host='127.0.0.1', port=6379, db=1,
                            password=None, parser_class=None,
                            unix_socket_path=None, connection_pool_class=None,
                            connection_pool_class_kwargs=None):
        connection_identifier = (host, port, db, parser_class, unix_socket_path, connection_pool_class)
        if not self._connection_pools.get(connection_identifier):
            connection_class = (
                unix_socket_path and UnixDomainSocketConnection or Connection
            )
            kwargs = {
                'db': db,
                'password': password,
                'connection_class': connection_class,
                'parser_class': parser_class,
            }
            kwargs.update(connection_pool_class_kwargs)
            if unix_socket_path is None:
                kwargs.update({
                    'host': host,
                    'port': port,
                })
            else:
                kwargs['path'] = unix_socket_path
            self._connection_pools[connection_identifier] = connection_pool_class(**kwargs)
        return self._connection_pools[connection_identifier]


pool = CacheConnectionPool()


class RedisClient(CacheClient):
    def __init__(self, server, params):
        """
        Connect to Redis, and set up cache backend.
        """
        self._init(server, params)

    def _init(self, server, params):
        super(RedisClient, self).__init__(params)
        self._server = server
        self._params = params

        unix_socket_path = None
        if ':' in self.server:
            host, port = self.server.rsplit(':', 1)
            try:
                port = int(port)
            except (ValueError, TypeError):
                raise ConfigError("port value must be an integer")
        else:
            host, port = None, None
            unix_socket_path = self.server

        kwargs = {
            'db': self.db,
            'password': self.password,
            'host': host,
            'port': port,
            'unix_socket_path': unix_socket_path,
        }

        connection_pool = pool.get_connection_pool(
            parser_class=self.parser_class,
            connection_pool_class=self.connection_pool_class,
            connection_pool_class_kwargs=self.connection_pool_class_kwargs,
            **kwargs
        )
        self._client = redis.StrictRedis(
            connection_pool=connection_pool,
            **kwargs
        )

    @property
    def server(self):
        return self._server or "127.0.0.1:6379"

    @property
    def params(self):
        return self._params or {}

    @property
    def options(self):
        return self.params.get('OPTIONS', {})

    @property
    def client(self):
        return self._client

    @property
    def connection_pool_class(self):
        cls = self.options.get('POOL_CLASS', 'redis.ConnectionPool')
        mod_path, cls_name = cls.rsplit('.', 1)
        try:
            mod = import_object(mod_path)
            pool_class = getattr(mod, cls_name)
        except (AttributeError, ImportError):
            raise ConfigError("Could not find connection pool class '%s'" % cls)
        return pool_class

    @property
    def connection_pool_class_kwargs(self):
        default = {
            'retry_on_timeout': False,
            'socket_keepalive': None,
            'socket_connect_timeout': 2,
            'socket_timeout': 3,
            'max_connections': 2 ** 10 * 10
        }
        kw = self.options.get('POOL_KWARGS', {})
        default.update(kw)
        return default


    @property
    def db(self):
        _db = self.params.get('db', self.options.get('DB', 1))
        try:
            _db = int(_db)
        except (ValueError, TypeError):
            raise ConfigError("db value must be an integer")
        return _db

    @property
    def password(self):
        return self.params.get('password', self.options.get('PASSWORD', None))

    @property
    def parser_class(self):
        cls = self.options.get('PARSER_CLASS', None)
        if cls is None:
            return DefaultParser
        mod_path, cls_name = cls.rsplit('.', 1)
        try:
            mod = import_object(mod_path)
            parser_class = getattr(mod, cls_name)
        except (AttributeError, ImportError):
            raise ConfigError("Could not find parser class '%s'" % cls)
        return parser_class

    def __getstate__(self):
        return {'params': self._params, 'server': self._server}

    def __setstate__(self, state):
        self._init(**state)

    def make_key(self, key, version=None):
        if not isinstance(key, CacheKey):
            key = CacheKey(super(RedisClient, self).make_key(key, version))
        return key

    def ping(self):
        self.client.ping()


class RedisCache(CacheMixin, RedisClient):
    def incr_version(self, key, delta=1, version=None):
        """
        Adds delta to the cache version for the supplied key. Returns the
        new version.

        Note: In Redis 2.0 you cannot rename a volitile key, so we have to move
        the value from the old key to the new key and maintain the ttl.
        """
        if version is None:
            version = self.version
        old_key = self.make_key(key, version)
        value = self.get(old_key, version=version)
        ttl = self._client.ttl(old_key)
        if value is None:
            raise ValueError("Key '%s' not found" % key)
        new_key = self.make_key(key, version=version + delta)
        # TODO: See if we can check the version of Redis, since 2.2 will be able
        # to rename volitile keys.
        self.set(new_key, value, timeout=ttl)
        self.delete(old_key)
        return version + delta

    def add(self, key, value, timeout=DEFAULT_TIMEOUT, version=None):
        """
        Add a value to the cache, failing if the key already exists.

        Returns ``True`` if the object was added, ``False`` if not.
        """
        return self.set(key, value, timeout, _add_only=True)

    def get(self, key, default=None, version=None):
        """
        Retrieve a value from the cache.

        Returns unpickled value if key is found, the default if not.
        """
        key = self.make_key(key, version=version)
        value = self._client.get(key)
        if value is None:
            return default
        result = self.unpickle(value)
        return result

    def _set(self, key, value, timeout, client, _add_only=False):
        if timeout is None or timeout == 0:
            if _add_only:
                return client.setnx(key, value)
            return client.set(key, value)
        elif timeout > 0:
            if _add_only:
                added = client.setnx(key, value)
                if added:
                    client.expire(key, timeout)
                return added
            return client.setex(key, timeout, value)
        else:
            return False

    def set(self, key, value, timeout=DEFAULT_TIMEOUT, version=None, client=None, _add_only=False):
        """
        Persist a value to the cache, and set an optional expiration time.
        """
        if not client:
            client = self._client
        key = self.make_key(key, version=version)
        if timeout is DEFAULT_TIMEOUT:
            timeout = self.default_timeout
        if timeout is not None:
            timeout = int(timeout)

        # If ``value`` is not an int, then pickle it
        if not isinstance(value, int) or isinstance(value, bool):
            result = self._set(key, pickle.dumps(value), timeout, client, _add_only)
        else:
            result = self._set(key, value, timeout, client, _add_only)
        # result is a boolean
        return result

    def delete(self, key, version=None):
        """
        Remove a key from the cache.
        """
        self._client.delete(self.make_key(key, version=version))

    def delete_many(self, keys, version=None):
        """
        Remove multiple keys at once.
        """
        if keys:
            keys = map(lambda key: self.make_key(key, version=version), keys)
            self._client.delete(*keys)

    def clear(self):
        """
        Flush all cache keys.
        """
        # TODO : potential data loss here, should we only delete keys based on the correct version ?
        self._client.flushdb()

    def unpickle(self, value):
        """
        Unpickles the given value.
        """
        if value and not isinstance(value, int) or isinstance(value, bool):
            return pickle.loads(value)
        return value

    def pickle(self, value):

        if value and not isinstance(value, int) or isinstance(value, bool):
            return pickle.dumps(value, pickle.HIGHEST_PROTOCOL)
        return value

    def get_many(self, keys, version=None):
        """
        Retrieve many keys.
        """
        if not keys:
            return {}
        recovered_data = SortedDict()
        new_keys = list(map(lambda key: self.make_key(key, version=version), keys))
        map_keys = dict(zip(new_keys, keys))
        results = self._client.mget(new_keys)
        for key, value in zip(new_keys, results):
            if value is None:
                continue
            value = self.unpickle(value)
            if isinstance(value, bytes_type):
                value = safestr(value)
            recovered_data[map_keys[key]] = value
        return recovered_data

    def set_many(self, data, timeout=DEFAULT_TIMEOUT, version=None):
        """
        Set a bunch of values in the cache at once from a dict of key/value
        pairs. This is much more efficient than calling set() multiple times.

        If timeout is given, that timeout will be used for the key; otherwise
        the default cache timeout will be used.
        """
        pipeline = self._client.pipeline()
        for key, value in data.items():
            self.set(key, value, timeout, version=version, client=pipeline)
        pipeline.execute()

    def incr(self, key, delta=1, version=None):
        """
        Add delta to value in the cache. If the key does not exist, raise a
        ValueError exception.
        """
        key = self.make_key(key, version=version)
        exists = self._client.exists(key)
        if not exists:
            raise ValueError("Key '%s' not found" % key)
        try:
            value = self._client.incr(key, delta)
        except redis.ResponseError:
            value = self.get(key) + delta
            self.set(key, value)
        return value

    def ttl(self, key, version=None):
        """
        Returns the 'time-to-live' of a key.  If the key is not volitile, i.e.
        it has not set expiration, then the value returned is None.  Otherwise,
        the value is the number of seconds remaining.  If the key does not exist,
        0 is returned.
        """
        key = self.make_key(key, version=version)
        if self._client.exists(key):
            return self._client.ttl(key)
        return 0

    def has_key(self, key, version=None):
        """
        Returns True if the key is in the cache and has not expired.
        """
        key = self.make_key(key, version=version)
        return self._client.exists(key)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
Torngas Whitelist Module

"""
from tornado import web
import types
from torngas.settings_manager import settings


def whitelisted(argument=None):
    """
    白名单，如果在参数中列出可访问的ip或在配置文件中列出，则被标记的请求方法仅可允许白名单ip访问
    :param argument: whitelist ip list
    :return:bool
    """

    def is_whitelisted(remote_ip, whitelist):
        if remote_ip in whitelist:
            return True
        else:
            return False

    if type(argument) is types.FunctionType:

        def wrapper(self, *args, **kwargs):
            white_setting = settings.WHITELIST
            if white_setting:
                if is_whitelisted(self.request.remote_ip,
                                  settings.WHITELIST):
                    return argument(self, *args, **kwargs)
                raise web.HTTPError(403)
            else:
                raise web.HTTPError(403)

        return wrapper

    else:
        if isinstance(argument, str):
            argument = [argument]

        elif not isinstance(argument, list):
            raise ValueError('whitelisted requires no parameters or '
                             'a string or list')

        def argument_wrapper(method):

            def validate(self, *args, **kwargs):
                if is_whitelisted(self.request.remote_ip, argument):
                    return method(self, *args, **kwargs)
                raise web.HTTPError(403)

            return validate

        return argument_wrapper



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
from jinja2 import Environment, FileSystemLoader, FileSystemBytecodeCache
from tornado.template import Loader
from torngas.settings_manager import settings
from jinja2.defaults import *
from jinja2.runtime import Undefined

_CACHE = FileSystemBytecodeCache()
_LOADER = FileSystemLoader([])
cfg = settings.TEMPLATE_CONFIG
_JINJA_ENV = Environment(bytecode_cache=_CACHE,
                         autoescape=cfg.get('autoescape', False),
                         cache_size=cfg.get('cache_size', 50),
                         auto_reload=cfg.get('filesystem_checks', True),
                         loader=_LOADER,
                         block_start_string=cfg.get('block_start_string', BLOCK_START_STRING),
                         block_end_string=cfg.get('block_end_string', BLOCK_END_STRING),
                         variable_start_string=cfg.get('variable_start_string', VARIABLE_START_STRING),
                         variable_end_string=cfg.get('variable_end_string', VARIABLE_END_STRING),
                         comment_start_string=cfg.get('comment_start_string', COMMENT_START_STRING),
                         comment_end_string=cfg.get('comment_end_string', COMMENT_END_STRING),
                         line_statement_prefix=cfg.get('line_statement_prefix', LINE_STATEMENT_PREFIX),
                         line_comment_prefix=cfg.get('line_comment_prefix', LINE_COMMENT_PREFIX),
                         trim_blocks=cfg.get('trim_blocks', TRIM_BLOCKS),
                         lstrip_blocks=cfg.get('lstrip_blocks', LSTRIP_BLOCKS),
                         newline_sequence=cfg.get('newline_sequence', NEWLINE_SEQUENCE),
                         keep_trailing_newline=cfg.get('keep_trailing_newline', KEEP_TRAILING_NEWLINE),
                         extensions=cfg.get('extensions', ()),
                         optimized=cfg.get('optimized', True),
                         undefined=cfg.get('undefined', Undefined),
                         finalize=cfg.get('finalize', None))


class Jinja2TemplateLoader(Loader):
    def __init__(self, root_directory='', **kwargs):
        super(Jinja2TemplateLoader, self).__init__(root_directory, **kwargs)
        path = os.path.abspath(root_directory)
        _JINJA_ENV.loader.searchpath = [path]

        cache_dir = os.path.abspath(settings.TEMPLATE_CONFIG.cache_directory)
        if not os.path.exists(cache_dir):
            os.makedirs(cache_dir)
        _CACHE.directory = cache_dir

    def load(self, name, parent_path=None):
        with self.lock:
            if os.path.isabs(name):
                path, file = os.path.split(name)
                _JINJA_ENV.loader.searchpath = [path]
                template = _JINJA_ENV.get_template(file)
            else:
                template = _JINJA_ENV.get_template(name)
            template.generate = template.render
            return template

    def reset(self):
        if hasattr(_JINJA_ENV, 'bytecode_cache') and _JINJA_ENV.bytecode_cache:
            _JINJA_ENV.bytecode_cache.clear()


# !/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import logging
import warnings
import logging.handlers
from tornado.log import LogFormatter
from ..settings_manager import settings
from tornado.util import import_object
from tornado.options import options
from torngas.global_settings import LOGGER
DEFAULT_FORMAT = '[%(levelname)1.1s %(asctime)s %(module)s:%(lineno)d] %(message)s'



class CustomRotatingFileHandler(logging.handlers.TimedRotatingFileHandler):
    def __init__(self, filename, when='h', interval=1, backupCount=20, encoding=None, delay=False, utc=False):
        dir_name, file_name = os.path.split(filename)
        if not dir_name:
            root_dir = settings.LOGGER_CONFIG['root_dir']
            filename = os.path.join(root_dir, file_name)

        super(CustomRotatingFileHandler, self).__init__(filename, when, interval, backupCount, encoding, delay,
                                                        utc)


class UsePortRotatingFileHandler(CustomRotatingFileHandler):
    def __init__(self, filename, when='h', interval=1, backupCount=20, encoding=None, delay=False, utc=False):
        try:
            if options.port:
                filename += '.%s' % options.port
        except:
            pass
        super(UsePortRotatingFileHandler, self).__init__(filename, when, interval, backupCount, encoding, delay,
                                                         utc)


class LoggerLoader(object):
    loggers = {}

    @classmethod
    def load_logger(cls):
        log_config = LOGGER
        if 'LOGGER' in settings:
            log_config.update(settings.LOGGER)

        for k, v in log_config.items():
            is_open = v.get('OPEN')
            logger = logging.getLogger(k)

            if is_open:
                cls.load_handler(logger, v)
                if not settings.DEBUG:
                    logger.propagate = 0
                else:
                    if options.log_to_stderr is False:
                        logger.propagate = 0
                    else:
                        logger.propagate = 1
                cls.loggers[k] = logger
            else:
                logger.disabled = True
                warnings.warn("logger %s is not open or not exist in logging config settings." % k)


    @classmethod
    def get_logger(cls, name):
        if name in cls.loggers:
            return cls.loggers[name]
        else:
            return logging.getLogger(name)


    @classmethod
    def load_handler(cls, logger, log_conf):
        global DEFAULT_FORMAT
        if settings.DEBUG:
            DEFAULT_FORMAT = LogFormatter.DEFAULT_FORMAT

        for handl in log_conf['HANDLERS']:
            module = handl.pop("module", None)
            level = handl.pop("level", None)

            if module:
                handler_module = import_object(module)
                handler = handler_module(**handl)
                log_formatter = LogFormatter(fmt=log_conf.get("FORMATTER", DEFAULT_FORMAT),
                                             color=settings.DEBUG)
                handler.setFormatter(log_formatter)
                if level:
                    handler.setLevel(level)
                logger.addHandler(handler)
                logger.setLevel(log_conf.get("LEVEL", "INFO"))




#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
from mako.lookup import TemplateLookup
from tornado.template import Loader
from torngas.settings_manager import settings

cfg = settings.TEMPLATE_CONFIG
_lookup = TemplateLookup(input_encoding=cfg.get('input_encoding', 'utf-8'),
                         output_encoding=cfg.get('output_encoding', 'utf-8'),
                         encoding_errors=cfg.get('encoding_errors', 'replace'),
                         filesystem_checks=cfg.get('filesystem_checks', True),
                         default_filters=cfg.get('default_filters', ['decode.utf8']),
                         collection_size=cfg.get('collection_size', -1),
                         format_exceptions=cfg.get('format_exceptions', False),
                         directories=cfg.get('directories', None),
                         module_directory=cfg.get('module_directory', None),
                         error_handler=cfg.get('error_handler', None),
                         disable_unicode=cfg.get('disable_unicode', False),
                         bytestring_passthrough=cfg.get('bytestring_passthrough', False),
                         cache_args=cfg.get('cache_args', False),
                         cache_impl=cfg.get('cache_impl', 'beaker'),
                         cache_enabled=cfg.get('cache_enabled', True),
                         cache_type=cfg.get('cache_type', None),
                         cache_dir=cfg.get('cache_dir', None),
                         cache_url=cfg.get('cache_url', None),
                         modulename_callable=cfg.get('modulename_callable', None),
                         module_writer=cfg.get('module_writer', None),
                         buffer_filters=cfg.get('buffer_filters', ()),
                         strict_undefined=cfg.get('strict_undefined', False),
                         imports=cfg.get('imports', None),
                         future_imports=cfg.get('future_imports', None),
                         enable_loop=cfg.get('enable_loop', True),
                         preprocessor=cfg.get('preprocessor', None),
                         lexer_cls=cfg.get('lexer_cls', None))


class MakoTemplateLoader(Loader):
    def __init__(self, root_directory, **kwargs):
        super(MakoTemplateLoader, self).__init__(root_directory, **kwargs)
        self.path = os.path.abspath(root_directory)
        self.module_directory = os.path.abspath(settings.TEMPLATE_CONFIG.cache_directory)

    def load(self, name, parent_path=None):
        with self.lock:
            _lookup.module_directory = self.module_directory
            _lookup.template_args['module_directory'] = self.module_directory
            if os.path.isabs(name):
                path, file = os.path.split(name)
                _lookup.directories = [path]

                template = _lookup.get_template(file)
            else:
                _lookup.directories = [self.path]

                template = _lookup.get_template(name)
        template.generate = template.render

        return template

    def reset(self):
        pass


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Multi-consumer multi-producer dispatching mechanism
Originally based on pydispatch (BSD) http://pypi.python.org/pypi/PyDispatcher/2.0.1
See license.txt for original license.
"""

from dispatcher import Signal, receiver


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
common handler,webhandler,apihandler
要获得torngas的中间件等特性需继承这些handler
"""
import json
from tornado.web import RequestHandler
from mixins.handler import HandlerMixin
from mixins.exception import UncaughtExceptionMixin
from exception import HttpBadRequestError, Http404


class WebHandler(UncaughtExceptionMixin, HandlerMixin, RequestHandler):
    def create_template_loader(self, template_path):
        loader = self.application.tmpl
        if loader is None:
            return super(WebHandler, self).create_template_loader(template_path)
        else:
            return loader(template_path)


class ApiHandler(UncaughtExceptionMixin, HandlerMixin, RequestHandler):
    def get_format(self, params_name="format"):
        format = self.get_argument(params_name, None)
        if not format:
            accept = self.request.headers.get('Accept')
            if accept:
                if 'javascript' in accept.lower():
                    format = 'jsonp'
                else:
                    format = 'json'
        else:
            format = format.lower()
        return format or 'json'

    def write_api(self, obj=None, nofail=False, ensure_ascii=True, fmt=None):
        if not obj:
            obj = {}
        if not fmt:
            fmt = self.get_format()

        if fmt == 'json':
            self.set_header("Content-Type", "application/json; charset=UTF-8")
            self.write(json.dumps(obj, ensure_ascii=ensure_ascii))
        elif fmt == 'jsonp':
            self.set_header("Content-Type", "application/javascript")
            callback = self.get_argument('callback', 'callback')
            self.write('%s(%s);' % (callback, json.dumps(obj, ensure_ascii=ensure_ascii)))
        elif nofail:
            self.write(obj)
        else:
            raise HttpBadRequestError('Unknown response format requested: %s' % format)


class ErrorHandler(UncaughtExceptionMixin, RequestHandler):
    def initialize(self, *args, **kwargs):
        pass

    def prepare(self):
        super(ErrorHandler, self).prepare()
        raise Http404()


#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
dbalchemy中间件，加入此中间件可以自动帮助dbalchemy模块处理连接的关闭
"""

from ..db.dbalchemy import Connector
from tornado.ioloop import PeriodicCallback
from sqlalchemy import exc
from sqlalchemy import event
from sqlalchemy.pool import Pool
from ..logger import SysLogger
from ..settings_manager import settings

connection = Connector.conn_pool


def connection_event():
    @event.listens_for(Pool, "checkout")
    def ping_connection(dbapi_connection, connection_record, connection_proxy):
        # 在每次从连接池获取一个连接时，首先测试连接池是否畅通
        # 如果不畅通，则断开重新建立连接，及时防丢
        cursor = dbapi_connection.cursor()
        try:
            cursor.execute("SELECT 1")
        except:
            SysLogger.error('database pool has gone away')
            raise exc.DisconnectionError()
        cursor.close()


def ping_db(conn_, ping_inteval):
    PeriodicCallback(conn_.ping_db, ping_inteval * 1000).start()


class DBAlchemyMiddleware(object):
    def process_init(self, application):

        connection_event()
        # 定时ping数据库，防止mysql go away，定时检测防丢
        interval = settings.PING_DB
        if interval > 0:
            for k, conn in connection.items():
                ping_db(conn, interval)

    def process_endcall(self, handler, clear):
        for k, conn in connection.items():
            if hasattr(conn, 'remove'):
                callable(conn.remove) and conn.remove()




# !/usr/bin/env python
# -*- coding: utf-8 -*-
import warnings
from tornado.web import url as urlspec
from tornado.util import import_object
from torngas.exception import UrlError


class Url(object):
    overall_kw = None
    prefix = None

    def __init__(self, prefix=None, **kwargs):
        self.prefix = prefix

        if kwargs:
            self.overall_kw = kwargs

    def __call__(self, pattern, handler, kwargs=None, name=None):
        kw = {}
        if self.overall_kw:
            kw.update(self.overall_kw)
        if kwargs:
            kw.update(kwargs)

        if self.prefix and isinstance(handler, str):
            handler = "%s.%s" % (self.prefix, handler)

        urlsp = urlspec(pattern,
                        handler,
                        kwargs=kw,
                        name=name)
        urlsp.repr_pattern = pattern
        return urlsp


def route(*pattens):
    urls = []
    for patt in pattens:
        if isinstance(patt, (tuple, list)):
            urls += list(patt)
        else:
            urls.append(patt)
    return urls


def include(pattern, handlers, prefix_name=None):
    """
    当你有多组url分布在不同的文件，或者你有多组url前缀不同,include可以帮你组织路由
    eg:
    url1.py:
        url=Url('handler_abc')
        URLS = route(
            url(r'/add/','add.AddHandler'),
            ...
        )
    url2.py:
        url=Url()
        URLS = route(
            url(r'/list/','handler_efg.list.ListHandler'),
            ...,
            include('/admin','url1.URLS')
        )

    最终的url可能类似于这样：
    url = [
        r'/list/','handler_efg.list.ListHandler',
        ...,
        r'/admin/add/' , 'handler_abc.add.AddHandler',
        ...

    ]
    app = Application(url)
    """
    try:
        if prefix_name:
            new_name = '%s-%s' % (prefix_name, '%s')
        else:
            new_name = '%s'
        if handlers and isinstance(handlers, str):
            handlers = import_object(handlers)
        else:
            handlers = handlers
        urlspecs = []
        if not pattern.endswith('/'):
            pattern += '/'

        if handlers and isinstance(handlers, (tuple, list)):
            for handler in handlers:
                if handler and isinstance(handler, urlspec):
                    patt = pattern + handler.repr_pattern \
                        .lstrip('^').lstrip('//').lstrip('/')
                    urlsp = urlspec(patt,
                                    handler.handler_class,
                                    kwargs=handler.kwargs,
                                    name=new_name % handler.name
                                    if handler.name else handler.name)

                    urlsp.repr_pattern = patt
                    urlspecs.append(urlsp)
            return urlspecs
        else:
            raise UrlError('url error,it is must be an tuple or list')
    except Exception:
        raise


# !/usr/bin/env python
# -*- coding: utf-8 -*-
import logging
import tornado.log
from functools import partial
from ..settings_manager import settings

ACCESS_LOGNAME = 'torngas.accesslog'
TRACE_LOGNAME = 'torngas.tracelog'
INFO_LOGNAME = 'torngas.infolog'


class _SysLogger(object):
    def __init__(self):
        self.root_logger = logging.getLogger('tornado')
        self.access_logger = logging.getLogger(ACCESS_LOGNAME)
        self.trace_logger = logging.getLogger(TRACE_LOGNAME)
        self.info_logger = logging.getLogger(INFO_LOGNAME)

    def parse_logger(self):
        self.root_logger.setLevel(settings.LOGGER_CONFIG['root_level'])

        if not settings.LOGGER_CONFIG['use_tornadolog']:
            from loggers import LoggerLoader

            LoggerLoader.load_logger()
            self.access_logger = LoggerLoader.get_logger(ACCESS_LOGNAME)
            self.trace_logger = LoggerLoader.get_logger(TRACE_LOGNAME)
            self.info_logger = LoggerLoader.get_logger(INFO_LOGNAME)
        else:
            import tornado.log

            self.access_logger = tornado.log.access_log
            self.trace_logger = tornado.log.app_log
            self.info_logger = tornado.log.gen_log


    @property
    def debug(self):
        """
        logging debug message
        """

        return partial(self.info_logger.debug)

    @property
    def info(self):
        """
        logging info message
        """
        return partial(self.info_logger.info)

    @property
    def warning(self):
        """
        logging warn message
        """
        return partial(self.trace_logger.warning)

    @property
    def error(self):
        """
        logging error message
        """
        return partial(self.trace_logger.error)

    @property
    def exception(self):
        """
        logging exception message
        """
        return partial(self.trace_logger.exception)


SysLogger = _SysLogger()


#!/usr/bin/env python
# -*- coding: utf-8  -*-

import sys
from tornado.util import import_object
from tornado.concurrent import Future
from tornado import ioloop

try:
    import futures
except ImportError:
    futures = None

if futures is None:
    FUTURES = Future
else:
    FUTURES = (futures.Future, Future)


def is_future(x):
    return isinstance(x, FUTURES)


import types
import contextlib
import itertools

try:
    import threading
except ImportError:
    import dummy_threading as threading

iters = [list, tuple]
PY2 = sys.version_info[0] == 2
PY3 = sys.version_info[0] == 3

if PY3:
    string_types = str,
    integer_types = int,
    class_types = type,
    text_type = str
    binary_type = bytes

    MAXSIZE = sys.maxsize
else:
    string_types = basestring,
    integer_types = (int, long)
    class_types = (type, types.ClassType)
    text_type = unicode
    binary_type = str


class RWLock(object):
    """
    Classic implementation of reader-writer lock with preference to writers.

    Readers can access a resource simultaneously.
    Writers get an exclusive access.

    API is self-descriptive:
        reader_enters()
        reader_leaves()
        writer_enters()
        writer_leaves()
    """

    def __init__(self):
        self.mutex = threading.RLock()
        self.can_read = threading.Semaphore(0)
        self.can_write = threading.Semaphore(0)
        self.active_readers = 0
        self.active_writers = 0
        self.waiting_readers = 0
        self.waiting_writers = 0

    def reader_enters(self):
        with self.mutex:
            if self.active_writers == 0 and self.waiting_writers == 0:
                self.active_readers += 1
                self.can_read.release()
            else:
                self.waiting_readers += 1
        self.can_read.acquire()

    def reader_leaves(self):
        with self.mutex:
            self.active_readers -= 1
            if self.active_readers == 0 and self.waiting_writers != 0:
                self.active_writers += 1
                self.waiting_writers -= 1
                self.can_write.release()

    @contextlib.contextmanager
    def reader(self):
        self.reader_enters()
        try:
            yield
        finally:
            self.reader_leaves()

    def writer_enters(self):
        with self.mutex:
            if self.active_writers == 0 and self.waiting_writers == 0 and self.active_readers == 0:
                self.active_writers += 1
                self.can_write.release()
            else:
                self.waiting_writers += 1
        self.can_write.acquire()

    def writer_leaves(self):
        with self.mutex:
            self.active_writers -= 1
            if self.waiting_writers != 0:
                self.active_writers += 1
                self.waiting_writers -= 1
                self.can_write.release()
            elif self.waiting_readers != 0:
                t = self.waiting_readers
                self.waiting_readers = 0
                self.active_readers += t
                while t > 0:
                    self.can_read.release()
                    t -= 1

    @contextlib.contextmanager
    def writer(self):
        self.writer_enters()
        try:
            yield
        finally:
            self.writer_leaves()


class cached_property(object):
    """
    Decorator that converts a method with a single self argument into a
    property cached on the instance.
    """

    def __init__(self, func):
        self.func = func

    def __get__(self, instance, type=None):
        if instance is None:
            return self
        res = instance.__dict__[self.func.__name__] = self.func(instance)
        return res


class LazyImport:
    """lazy import module"""

    def __init__(self, module_name):
        self.module_name = module_name
        self.module = None

    def __getattr__(self, func_name):
        if self.module is None:
            self.module = import_object(self.module_name)
        return getattr(self.module, func_name)


lazyimport = LazyImport


class Null(object):
    def __new__(cls, *args, **kwargs):
        if '_instance' not in vars(cls):
            cls._instance = super(Null, cls).__new__(cls, *args, **kwargs)
        return cls._instance

    def __init__(self, *args, **kwargs): pass

    def __call__(self, *args, **kwargs): return self

    def __repr__(self): return ""

    def __nonzero__(self): return False

    def __getattr__(self, item): return self

    def __setattr__(self, key, value): return self

    def __delattr__(self, item): return self

    def __len__(self): return 0

    def __iter__(self): return iter(())

    def __getitem__(self, item): return self

    def __delitem__(self, key): return self

    def __setitem__(self, key, value): return self


def safeunicode(obj, encoding='utf-8'):
    """
    Converts any given object to unicode string.

        >>> safeunicode('hello')
        u'hello'
        >>> safeunicode(2)
        u'2'
        >>> safeunicode('\xe1\x88\xb4')
        u'\u1234'
    """
    t = type(obj)
    if t is unicode:
        return obj
    elif t is str:
        return obj.decode(encoding)
    elif t in [int, float, bool]:
        return unicode(obj)
    elif hasattr(obj, '__unicode__') or isinstance(obj, unicode):
        return unicode(obj)
    else:
        return str(obj).decode(encoding)


def safestr(obj, encoding='utf-8'):
    r"""
    Converts any given object to utf-8 encoded string.

        >>> safestr('hello')
        'hello'
        >>> safestr(u'\u1234')
        '\xe1\x88\xb4'
        >>> safestr(2)
        '2'
    """
    if isinstance(obj, unicode):
        return obj.encode(encoding)
    elif isinstance(obj, str):
        return obj
    elif hasattr(obj, 'next'):  # iterator
        return itertools.imap(safestr, obj)
    else:
        return str(obj)

# for backward-compatibility
utf8 = safestr


def _strips(direction, text, remove):
    if isinstance(remove, iters):
        for subr in remove:
            text = _strips(direction, text, subr)
        return text

    if direction == 'l':
        if text.startswith(remove):
            return text[len(remove):]
    elif direction == 'r':
        if text.endswith(remove):
            return text[:-len(remove)]
    else:
        raise ValueError, "Direction needs to be r or l."
    return text


def rstrips(text, remove):
    """
    removes the string `remove` from the right of `text`

        >>> rstrips("foobar", "bar")
        'foo'

    """
    return _strips('r', text, remove)


def lstrips(text, remove):
    """
    removes the string `remove` from the left of `text`

        >>> lstrips("foobar", "foo")
        'bar'
        >>> lstrips('http://foo.org/', ['http://', 'https://'])
        'foo.org/'
        >>> lstrips('FOOBARBAZ', ['FOO', 'BAR'])
        'BAZ'
        >>> lstrips('FOOBARBAZ', ['BAR', 'FOO'])
        'BARBAZ'

    """
    return _strips('l', text, remove)


def strips(text, remove):
    """
    removes the string `remove` from the both sides of `text`

        >>> strips("foobarfoo", "foo")
        'bar'

    """
    return rstrips(lstrips(text, remove), remove)


class IterBetter:
    """
    Returns an object that can be used as an iterator
    but can also be used via __getitem__ (although it
    cannot go backwards -- that is, you cannot request
    `iterbetter[0]` after requesting `iterbetter[1]`).

        >>> import itertools
        >>> c = iterbetter(itertools.count())
        >>> c[1]
        1
        >>> c[5]
        5
        >>> c[3]
        Traceback (most recent call last):
            ...
        IndexError: already passed 3

    For boolean test, IterBetter peeps at first value in the itertor without effecting the iteration.

        >>> c = iterbetter(iter(range(5)))
        >>> bool(c)
        True
        >>> list(c)
        [0, 1, 2, 3, 4]
        >>> c = iterbetter(iter([]))
        >>> bool(c)
        False
        >>> list(c)
        []
    """

    def __init__(self, iterator):
        self.i, self.c = iterator, 0

    def __iter__(self):
        if hasattr(self, "_head"):
            yield self._head

        while 1:
            yield self.i.next()
            self.c += 1

    def __getitem__(self, i):
        # todo: slices
        if i < self.c:
            raise IndexError, "already passed " + str(i)
        try:
            while i > self.c:
                self.i.next()
                self.c += 1
                # now self.c == i
            self.c += 1
            return self.i.next()
        except StopIteration:
            raise IndexError, str(i)

    def __nonzero__(self):
        if hasattr(self, "__len__"):
            return len(self) != 0
        elif hasattr(self, "_head"):
            return True
        else:
            try:
                self._head = self.i.next()
            except StopIteration:
                return False
            else:
                return True


iterbetter = IterBetter


def sleep(seconds):
    """
    async sleep .
    yield sleep(5)
    :param seconds:
    :return:
    """
    future = Future()
    ioloop.IOLoop.current().call_later(seconds, lambda: future.set_result(None))
    return future


# !/usr/bin/env python
# -*- coding: utf-8 -*-
import threading
class BaseHttpModule(object):
    """
    编写httpmodule需继承BaseHttpModule并实现其中任何一个方法即可

    """
    _instance_lock = threading.Lock()
    __instance = None

    def __new__(cls, *args, **kwargs):
        if not cls.__instance:
            with cls._instance_lock:
                cls.__instance = super(BaseHttpModule, cls).__new__(
                    cls, *args, **kwargs)

        return cls.__instance

    def begin_request(self, handler, clear):
        """
        匹配路由后，执行处理handler时调用,你可以在这里对request进行过滤后finish request
        :param handler: handler对象
        v1.4.2:支持异步
        """
        pass


    def begin_render(self, handler, clear, template_name, **kwargs):
        """

        :param handler: 返回handler对象
        :param template_name: 模板名称
        :param kwargs: 模板变量
        """
        pass

    def begin_response(self, handler, clear, chunk=None):
        """
        请求结束后响应时调用，此事件在finish之前,render之后被调用，
        这里你可以对响应数据做最后的处理
        ***切记，这里不能再次调用finish，否则会发生循环调用导致错误***
        :param handler: handler对象
        """
        pass

    def complete_response(self, handler, clear):
        """
        请求呈现客户端后调用
        :param handler: handler对象
        """
        pass




#!/usr/bin/env python
# -*- coding: utf-8 -*-
import re
from hashlib import md5
from tornado.util import import_object
from tornado.web import ErrorHandler
from torngas.settings_manager import settings
from tornado import gen
from torngas.exception import BaseError
from . import BaseHttpModule

try:
    from tornado.concurrent import is_future
except ImportError:
    from torngas.utils import is_future

EXCLUDE_PREFIX = '!'


class HttpModuleMiddleware(object):
    common_modules = []
    route_modules = {}
    named_handlers = None
    non_executes_modules = {}
    re_complies = {}

    def _execute_module(self, handler, clear, module, method, name=None, **kwargs):
        try:
            def run_method_():
                if method.__name__ == "begin_response":
                    return method(handler, clear, kwargs.pop("chunk", None))
                elif method.__name__ == "begin_render":
                    return method(handler, clear, **kwargs)
                elif method.__name__ == 'begin_request':
                    return method(handler, clear)
                elif method.__name__ == 'complete_response':
                    return method(handler, clear)

            if name:
                if hasattr(handler, 'pattern__'):
                    patt = handler.pattern__

                    if name == patt:
                        url_spec = self.named_handlers.get(name, None)
                        if url_spec:
                            if isinstance(handler, url_spec.handler_class):
                                return run_method_()
                    elif handler.request.re_match_table[name]:
                        return run_method_()

            else:
                pattern__ = getattr(handler, 'pattern__', None)

                def check_(key):
                    non_execute = self.non_executes_modules.get(key, [])
                    if non_execute:
                        for n in non_execute:
                            if isinstance(module, n):
                                return True

                if pattern__ in self.non_executes_modules:
                    if check_(handler.pattern__): return

                if hasattr(handler.request, 're_match_table') and handler.request.re_match_table:
                    for key, match in handler.request.re_match_table.items():
                        if match:
                            if check_(key): return

                if not isinstance(handler, ErrorHandler):
                    return run_method_()

        except BaseException, ex:
            raise

    def _class_wrap(self, handler_class, name, url):
        # 防止使用同一个handler的路由出现错误，pattern__会被相同的handler的被覆盖
        scope = {}
        class_prefix = md5("%s_%s_%s" % (handler_class.__name__, name, url,)).hexdigest()
        class_name = "%s_%s" % (handler_class.__name__, class_prefix,)
        scope['old_' + class_name] = handler_class
        exec "class %s(old_%s):pass" % (class_name, class_name,) in scope
        class_ = scope[class_name]
        return class_

    def process_init(self, application):
        self.named_handlers = application.named_handlers
        # 为每个载入app的路由设定路由名称

        def check_baseclass_(cls):

            if BaseHttpModule not in cls.__bases__:
                raise BaseError("http_module '%s' must inherit from the \
                Basemodule" % str(cls))

        for k, urlspec in self.named_handlers.items():
            urlspec.handler_class = self._class_wrap(urlspec.handler_class, k, urlspec.repr_pattern)
            urlspec.handler_class.pattern__ = k

        # 通用module载入
        c_modules = settings.COMMON_MODULES
        if c_modules:
            for module in c_modules:
                try:
                    import_m = import_object(module)
                    check_baseclass_(import_m)
                    m = import_m()

                except ImportError:
                    raise
                if m not in self.common_modules:
                    self.common_modules.append(m)
        # 路由级module载入
        r_modules = settings.ROUTE_MODULES
        if r_modules:
            def choice_module_(m):
                if m.startswith("%sall" % EXCLUDE_PREFIX):
                    if c_modules:
                        for cs in self.common_modules:
                            cls_type = cs.__class__
                            if cls_type not in non_modules:
                                non_modules.append(cls_type)

                elif m.startswith(EXCLUDE_PREFIX):
                    import_m = import_object(m.lstrip(EXCLUDE_PREFIX))
                    check_baseclass_(import_m)
                    if import_m not in non_modules:
                        non_modules.append(import_m)
                else:
                    import_m = import_object(m)
                    check_baseclass_(import_m)
                    inst_import_m = import_m()
                    if inst_import_m not in modules_lst:
                        modules_lst.append(inst_import_m)

            for name, r_mods in r_modules.items():
                try:
                    modules_lst = []
                    non_modules = []

                    [choice_module_(m) for m in r_mods]
                    if non_modules:
                        if name not in self.non_executes_modules:
                            self.non_executes_modules[name] = non_modules
                    if name not in self.route_modules:
                        self.route_modules[name] = modules_lst
                    self.re_complies[name] = re.compile(name)
                except ImportError:
                    raise

    @gen.coroutine
    def _do_all_execute(self, handler, clear, method_name, **kwargs):

        for c_module in self.common_modules:
            result = self._execute_module(handler, clear, c_module, getattr(c_module, method_name), **kwargs)
            if is_future(result):
                result = yield result
            if result:
                raise gen.Return(1)

        for name, r_module in self.route_modules.items():
            for md in r_module:
                result = self._execute_module(handler, clear, md, getattr(md, method_name), name, **kwargs)
                if is_future(result):
                    result = yield result
                if result:
                    raise gen.Return(1)

    def process_call(self, request, clear):
        re_matchs = {}
        for key, compil in self.re_complies.items():
            re_matchs[key] = compil.search(request.path)

        request.re_match_table = re_matchs

    def process_request(self, handler, clear):
        return self._do_all_execute(handler, clear, 'begin_request')

    def process_response(self, handler, clear, chunk):
        return self._do_all_execute(handler, clear, 'begin_response', chunk=chunk)

    def process_render(self, handler, clear, template_name, **kwargs):
        return self._do_all_execute(handler, clear, 'begin_render', template_name=template_name, **kwargs)

    def process_endcall(self, handler, clear):
        return self._do_all_execute(handler, clear, 'complete_response')


#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import warnings

from tornado.util import import_object
from tornado.options import options

from exception import ConfigError
from torngas.storage import storage
from torngas import global_settings

SETTINGS_MODULE_ENVIRON = "TORNGAS_APP_SETTINGS"


class _Settings(object):
    def __contains__(self, item):
        setting = _Settings.settings_object()
        return hasattr(setting, item)

    def __getattr__(self, item):
        setting = _Settings.settings_object()
        if hasattr(setting, item):
            config = getattr(setting, item)
        else:
            raise ConfigError('settings "%s" not exist!' % item)

        return storage(config) if type(config) is dict else config

    @classmethod
    def settings_object(cls):

        if not hasattr(cls, '_sett'):
            cls._sett = global_settings
            try:
                sett_obj = import_object(options.settings)
                cls._sett.__dict__.update(sett_obj.__dict__)
            except AttributeError:
                if os.environ.get(SETTINGS_MODULE_ENVIRON, None):
                    sett_obj = import_object(os.environ[SETTINGS_MODULE_ENVIRON])
                    cls._sett.__dict__.update(sett_obj.__dict__)
                else:
                    raise ConfigError(
                        'tornado.options not have "settings",You may try to use settings before "define settings module"')
            except ImportError:
                cls._sett = global_settings
                warnings.warn(
                    'settings file import error.')

        return cls._sett


settings = _Settings()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import warnings
import sys
import tornado.httpserver
import tornado.ioloop
import tornado.options
import tornado.web
from tornado.options import define, options, parse_command_line
from tornado.util import import_object
from exception import ConfigError, ArgumentError, UrlError
from application import Application
from settings_manager import settings
from logger import SysLogger

reload(sys)
sys.setdefaultencoding('utf-8')


class Server(object):
    def __init__(self, ioloop=None):
        self.urls = []
        self.application = None
        self.httpserver = None
        self.ioloop = ioloop
        self.is_parselogger = False

    def _patch_httpserver(self):
        """
        重写httpserver的xheader配置，让gunicorn可以加载xheaders设置
        :return:
        """
        httpserver = sys.modules["tornado.httpserver"]
        try:
            xhs = settings.XHEADERS
        except:
            xhs = True

        class TorngasHTTPServer(httpserver.HTTPServer):
            def __init__(self, request_callback, xheaders=xhs, **kwargs):
                super(TorngasHTTPServer, self).__init__(request_callback,
                                                        xheaders=xheaders,
                                                        **kwargs)


        httpserver.HTTPServer = TorngasHTTPServer
        sys.modules["tornado.httpserver"] = httpserver

    def load_application(self, application=None):
        """

        :type application: torngas.application.Application subclass or instance
        :return:
        """
        self._patch_httpserver()
        if settings.TRANSLATIONS:
            try:
                from tornado import locale

                locale.load_translations(settings.TRANSLATIONS_CONF.translations_dir)
            except:
                warnings.warn('locale dir load failure,maybe your config file is not set correctly.')

        if not application:
            self._install_application(application)
        elif isinstance(application, Application):
            self.application = application
        elif issubclass(application, Application):
            self._install_application(application)
        else:
            raise ArgumentError('need torngas.application.Application instance object or subclass.')

        tmpl = settings.TEMPLATE_CONFIG.template_engine
        self.application.tmpl = import_object(tmpl) if tmpl else None

        return self.application

    def _install_application(self, application):
        if not self.urls:
            raise UrlError("urls not found.")
        if application:
            app_class = application
        else:
            app_class = Application

        tornado_conf = settings.TORNADO_CONF
        if 'default_handler_class' in tornado_conf and \
                isinstance(tornado_conf['default_handler_class'], basestring):
            tornado_conf['default_handler_class'] = import_object(tornado_conf['default_handler_class'])

        else:
            tornado_conf['default_handler_class'] = import_object('torngas.handler.ErrorHandler')

        tornado_conf['debug'] = settings.DEBUG
        self.application = app_class(handlers=self.urls,
                                     default_host='',
                                     transforms=None, wsgi=False,
                                     middlewares=settings.MIDDLEWARE_CLASSES,
                                     **tornado_conf)

    def load_urls(self):
        urls = []
        if settings.INSTALLED_APPS:
            for app_name in settings.INSTALLED_APPS:
                app_urls = import_object(app_name + '.urls.urls')
                urls.extend(app_urls)
        else:
            raise ConfigError('load urls error,INSTALLED_APPS not found!')
        self.urls = urls
        return self.urls

    def load_httpserver(self, sockets=None, **kwargs):
        if not sockets:
            from tornado.netutil import bind_sockets

            if settings.IPV4_ONLY:
                import socket

                sockets = bind_sockets(options.port, options.address, family=socket.AF_INET)
            else:
                sockets = bind_sockets(options.port, options.address)

        http_server = tornado.httpserver.HTTPServer(self.application, **kwargs)

        http_server.add_sockets(sockets)
        self.httpserver = http_server
        return self.httpserver

    def server_start(self, sockets=None, **kwargs):
        if not self.httpserver:
            self.load_httpserver(sockets, **kwargs)

        self.start()

    def load_all(self, application=None, sockets=None, **kwargs):
        self.parse_command()
        self.load_urls()
        self.load_application(application)
        if not self.httpserver:
            self.load_httpserver(sockets, **kwargs)

    def start(self):
        if not self.is_parselogger:
            self.parse_logger()

        self.print_settings_info()

        if not self.ioloop:
            self.ioloop = tornado.ioloop.IOLoop.instance()

        self.ioloop.start()

    def print_settings_info(self):
        if settings.DEBUG:
            print 'tornado version: %s' % tornado.version
            print 'load middleware:'
            for middl in settings.MIDDLEWARE_CLASSES:
                print ' -', str(middl)
            print 'locale support: %s' % settings.TRANSLATIONS
            print 'load apps:'
            for app in settings.INSTALLED_APPS:
                print ' -', str(app)
            print 'IPV4_Only: %s' % settings.IPV4_ONLY
            print 'template engine: %s' % settings.TEMPLATE_CONFIG.template_engine
            print 'server started. development server at http://%s:%s/' % (options.address, options.port)

    def parse_command(self, args=None, final=True):
        """
        解析命令行参数，解析logger配置
        :return:
        """
        self.define()
        parse_command_line(args, final)
        self.parse_logger()

    def define(self):
        """
        定义命令行参数,你可以自定义很多自己的命令行参数，或重写此方法覆盖默认参数
        :return:
        """
        define("port", default=8000, help="run server on it", type=int)
        define("settings", help="setting module name", type=str)
        define("address", default='0.0.0.0', help='listen host,default:0.0.0.0', type=str)

    def parse_logger(self):
        """
        加载logger配置，如果使用gunicorn，你需要手动调用加载
        :return:
        """
        SysLogger.parse_logger()
        self.is_parselogger = True


def run(application=None, sockets=None, **kwargs):
    server = Server()
    server.load_all(application, sockets, **kwargs)
    server.start()


#!/usr/bin/env python
# -*- coding: utf-8  -*-
# Created by mqingyn on 2015/1/16.
from dispatch import Signal

call_started = Signal(providing_args=["request"])
handler_started = Signal(providing_args=["handler"])
handler_response = Signal(providing_args=["handler", "chunk"])
call_finished = Signal(providing_args=["handler"])
handler_render = Signal(providing_args=["handler", "template_name", "kwargs"])


#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
线程future，此函数依赖tornado版本>3.0
如果某些函数耗时较长，且不提供异步回调的功能，例如同步调用mysql，执行一个耗时命令并返回结果，
这些函数无法利用tornado的async特性，此处利用线程池模拟了一个异步方法，可以防止同步耗时操作阻塞进程，
只要在需要异步的方法上加@async_execute装饰器，则该函数便可以支持
@asynchronous或gen的非阻塞方式调用

依赖：该方法必须安装futures模块才可以使用
例子：
    
class Test(Base):
    @coroutine
    def get(self):
        a=''
        b=''
        #支持使用gen的task模块来同步化异步调用
        result = yield self.dosomething(a, b)
        self.finish(result)

    @async_execute
    def dosomething(self,a,b):
        # 这里可能耗时很久
        #callback参数不会在这里有任何调用，这个耗时方法本身并没有异步，
        #同步的方法无论如何都不会毫无代价的变成异步，
        #该装饰器为此模拟了异步操作，但注意：这是用线程池模拟的
        # something...
        result='return'
        return result

class Test(Base):
    @asynchronous
    def get(self):
        a=''
        b=''
        #这是用tornado标准的异步回调方式
        self.dosomething(a,b,self.callback)

    def callback(self,result):
        #回调函数
        self.finish(result)

    @async_execute
    def dosomething(self,a,b,callback=None):
        #耗时操作
        result='return'
        return result
"""


import functools
from tornado.netutil import Resolver, ThreadedResolver
from tornado.ioloop import IOLoop

Resolver.configure('tornado.netutil.ThreadedResolver', num_threads=10)

def async_execute(fn):
    """
    新版tornado已有这个函数的实现，但是需要在class中绑定self.ioloop和self.executor,不太方便，这个版本消除了这个问题
    """
    thread_resolver = ThreadedResolver()

    @functools.wraps(fn)
    def wrapper(self, *args, **kwargs):
        callback = kwargs.pop("callback", None)
        future = thread_resolver.executor.submit(fn, self, *args, **kwargs)
        if callback:
            IOLoop.current().add_future(future,
                                    lambda future: callback(future.result()))
        return future
    return wrapper


if __name__ == '__main__':
    pass



#!/usr/bin/env python
# -*- coding: utf-8 -*-
__author__ = 'mqingyn'
__version__ = '1.7.1'

version = tuple(map(int, __version__.split('.')))

from settings_manager import settings
from webserver import Server, run
from exception import ConfigError, ArgumentError
from urlhelper import Url, route, include
from utils import is_future, RWLock, cached_property, lazyimport, Null, \
    safestr, safeunicode, strips, iterbetter, sleep
from storage import storage, storify, sorteddict, ThreadedDict



#!/usr/bin/env python
# -*- coding: utf-8  -*-
"""
SignalMiddleware提供在程序运行至中间件call,request,response,endcall,render几个阶段时触发信号的能力

"""
from torngas.signal import *


class SignalMiddleware(object):
    def process_call(self, request, clear):
        call_started.send(sender=request.__class__, request=request)

    def process_request(self, handler, clear):
        handler_started.send(sender=handler.__class__, handler=handler)

    def process_response(self, handler, clear, chunk):
        handler_response.send(sender=handler.__class__, handler=handler, chunk=chunk)

    def process_endcall(self, handler, clear):
        call_finished.send(sender=handler.__class__, handler=handler)

    def process_render(self, handler, clear, template_name, **kwargs):
        handler_render.send(sender=handler.__class__, handler=handler, template_name=template_name, kwargs=kwargs)



# !/usr/bin/env python
# -*- coding: utf-8 -*-
"""
基于sqlalchemy的数据库组件，提供全局连接，配置管理，主从库支持，目前支持单主多从
connection config eg:
DATABASE_CONNECTION = {
    'default': {
        'connections': [{
                            'ROLE': 'master',
                            'DRIVER': 'mysql+mysqldb',
                            'UID': 'root',
                            'PASSWD': '',
                            'HOST': '',
                            'PORT': 3306,
                            'DATABASE': '',
                            'QUERY': {"charset": "utf8"}

                        },
                        {
                            'ROLE': 'slave',
                            'DRIVER': 'mysql+mysqldb',
                            'UID': 'root',
                            'PASSWD': '',
                            'HOST': '',
                            'PORT': 3306,
                            'DATABASE': '',
                            'QUERY': {"charset": "utf8"}

                        }]
    }
}
"""
import random
import threading
from math import ceil
from ..settings_manager import settings
from ..exception import ConfigError
from ..storage import storage
from ..utils import string_types
from sqlalchemy import engine_from_config
from sqlalchemy.orm import scoped_session, sessionmaker, Query
from sqlalchemy.engine import url
from ..logger import SysLogger
from tornado.util import import_object
from sqlalchemy.ext.declarative import declarative_base, declared_attr
from ..decorators.async_execute import async_execute

__all__ = [
    'MetaBaseModel',
    'create_session',
    'DBConfigParser',
    'Model',
    'ConnBase',
    'SQLAlchemy',
    'get_connector',
    'Connector'
]
MetaBaseModel = declarative_base()

_SQLALCHEMY_PREFIX = 'sqlalchemy.'
_CONNECTION_TYPE = (
    'master',
    'slave',
)

_BASE_SQLALCHEMY_CONFIGURATION = {
    'sqlalchemy.connect_args': {},
    'sqlalchemy.echo': False,
    'sqlalchemy.max_overflow': 10,
    'sqlalchemy.echo_pool': False,
    'sqlalchemy.pool_timeout': 5,
    'sqlalchemy.encoding': 'utf-8',
    'sqlalchemy.pool_size': 10,
    'sqlalchemy.poolclass': 'QueuePool'
}


def create_session(engine=None, scopefunc=None, twophase=False, **kwargs):
    def import_(param):
        if isinstance(param, string_types):
            return import_object(param)
        else:
            return param

    if scopefunc:
        scopefunc = import_(scopefunc)

    if 'extension' in kwargs:
        kwargs['extension'] = import_(kwargs['extension'])

    if 'class_' in kwargs:
        kwargs['class_'] = import_(kwargs['class_'])

    if 'query_cls' in kwargs:
        kwargs['query_cls'] = import_(kwargs['query_cls'])

    session = sessionmaker(autoflush=kwargs.pop('autoflush', True),
                           autocommit=kwargs.pop('autocommit', False),
                           expire_on_commit=kwargs.pop('expire_on_commit', True),
                           info=kwargs.pop('expire_on_commit', None),
                           **kwargs)

    if not twophase:
        session.configure(bind=engine)

    else:
        # eg:binds = {User:engine1, Account:engine2}
        session.configure(binds=engine, twophase=twophase)

    return scoped_session(session, scopefunc)


class DBConfigParser(object):
    @classmethod
    def parser_engines(cls):

        connections = settings.DATABASE_CONNECTION
        engines = {}
        for connection_name, connection_item in connections.items():
            if connection_name in engines:
                raise ConfigError('conn:%s ,has already exist.' % connection_name)

            engines[connection_name] = {
                'kwargs': connection_item.get('kwargs', {}),
                'master': [],
                'slaves': [],
            }

            connections_str = connection_item['connections']

            for conn in connections_str:
                dburl = url.URL(drivername=conn['DRIVER']
                                , username=conn['UID']
                                , password=conn['PASSWD']
                                , host=conn['HOST']
                                , port=conn['PORT']
                                , database=conn['DATABASE']
                                , query=conn['QUERY'])
                if conn['ROLE'] == _CONNECTION_TYPE[0]:
                    engines[connection_name]['master'].append(dburl)
                elif conn['ROLE'] == _CONNECTION_TYPE[1]:
                    engines[connection_name]['slaves'].append(dburl)
                else:
                    raise ConfigError('role %s not allowed.' % conn['ROLE'])

            if not len(engines[connection_name]['master']):
                raise ConfigError('conn:%s ,master connection not found.' % connection_name)

        return engines

    @classmethod
    def parser_sqlalchemy_conf(cls):
        try:
            config = settings.SQLALCHEMY_CONFIGURATION
        except ConfigError, ex:
            SysLogger.warning("SQLALCHEMY_CONFIGURATION not found,using default sqlalchemy configuration")
            config = _BASE_SQLALCHEMY_CONFIGURATION

        poolclass_conf = _SQLALCHEMY_PREFIX + 'poolclass'
        poolclass = config[poolclass_conf] if poolclass_conf in config else None

        if poolclass:
            config[poolclass_conf] = import_object('sqlalchemy.pool.' + poolclass)
        return config


class _Connector(object):
    _conn_lock = threading.Lock()
    _conn = None

    def __init__(self, connection_class):
        self.class_ = connection_class

    @property
    def conn_pool(self):
        if _Connector._conn:
            return _Connector._conn
        else:
            with _Connector._conn_lock:
                connection_pool = storage()
                engines = DBConfigParser.parser_engines()
                config = DBConfigParser.parser_sqlalchemy_conf()
                try:
                    for name, engine in engines.items():
                        kw = engine['kwargs']
                        connection_pool[name] = self.class_(config,
                                                            master_url=engine['master'],
                                                            slaves_url=engine['slaves'],
                                                            session_conf=kw.pop('session', {}),
                                                            **kw)
                except Exception:
                    raise

            _Connector._conn = connection_pool
            return _Connector._conn

    def get_conn(self, conn_name):
        return self.conn_pool[conn_name]

    def get_session(self, connection_name='default'):
        session = self.get_conn(connection_name).session
        return session


class BaseQuery(Query):
    def paginate(self, page, per_page=20, default=None):
        """Returns `per_page` items from page `page`.
        Returns an :class:`Pagination` object.
        """
        if page < 1:
            return default
        items = self.limit(per_page).offset((page - 1) * per_page).all()
        if not items and page != 1:
            return default

        # No need to count if we're on the first page and there are fewer
        # items than we expected.
        if page == 1 and len(items) < per_page:
            total = len(items)
        else:
            total = self.order_by(None).count()

        return Pagination(self, page, per_page, total, items)


class Model(MetaBaseModel):
    __abstract__ = True
    __tablename__ = ''
    __table_args__ = {
        'mysql_engine': 'InnoDB',
        'mysql_charset': 'utf8'
    }
    __connection_name__ = 'default'

    @declared_attr
    def Q(cls):
        return Connector.get_conn(cls.__connection_name__).query()

    @declared_attr
    def session(cls):
        slave = Connector.get_session(cls.__connection_name__)['slave']

        slave.using_master = lambda: \
            Connector.get_session(cls.__connection_name__)['master']
        return slave


    def __init__(self, **kwargs):
        for k, v in kwargs.iteritems():
            setattr(self, k, v)


class ConnBase(object):
    def __init__(self, config=None, master_url=None, slaves_url=None, session_conf=None, **kwargs):
        self.kwargs = kwargs or {}
        self.config = config or {}
        self.master_url = master_url
        self.slaves_url = slaves_url or []
        self.session_conf = session_conf or {}

    @property
    def engines(self):
        raise NotImplementedError

    @property
    def session(self):
        raise NotImplementedError

    def query(self, using_master=False):
        raise NotImplementedError

    def remove(self):
        """
        remove session to pool
        :return:
        """

    def ping_db(self):
        """
        ping db ,to prevent mysql sql gone away
        :return:
        """
        pass


class SQLAlchemy(ConnBase):
    def __init__(self, config=None, master_url=None, slaves_url=None, session_conf=None, **kwargs):

        """
        连接对象init
        :param base_conf: 全局配置，sqlalchemy.开头
        :param master_url: 主库连接
        :param slaves_url: 从库连接
        :param others_url: 其他类型，暂未实现
        :param kwargs: 自定义配置
        """
        super(SQLAlchemy, self).__init__(config=config, master_url=master_url, slaves_url=slaves_url,
                                         session_conf=session_conf, **kwargs)
        # 每一个engine都会有一个factory，感觉挺闹心，要是有10个engine。。就得10个factory，未来寻找一下全局只有一个factory的方案
        self._slave_tmp = None
        session_conf['query_cls'] = BaseQuery
        self._master_engine = engine_from_config(self.config, prefix=_SQLALCHEMY_PREFIX, url=master_url[0], **kwargs)
        self._master_session = create_session(self._master_engine, **session_conf)
        self._slaves_session = []
        self._slave_engine = []

        for slave in slaves_url:
            slave_engine = engine_from_config(self.config, prefix=_SQLALCHEMY_PREFIX, url=slave, **kwargs)
            self._slave_engine.append(slave_engine)
            self._slaves_session.append(create_session(slave_engine, **session_conf))

    def remove(self):
        # 连接释放通过可通过中间件提供，用户可自行决定是否要使用
        self._master_session.remove()
        if self._slaves_session:
            for slave in self._slaves_session:
                slave.remove()
        self._slave_tmp = None

    @property
    def session(self):
        return {
            'master': self.using_master_session(),
            'slave': self.using_slave_session()
        }

    def using_master_session(self):
        """
        主库session
        """
        return self._master_session

    @property
    def engines(self):
        return {
            'master': self._master_engine,
            'slaves': self._slave_engine
        }

    def using_slave_session(self):
        """
        从库session
        """
        if not self._slave_tmp:
            if self._slaves_session:
                self._slave_tmp = random.choice(self._slaves_session)
            else:
                self._slave_tmp = self._master_session

        return self._slave_tmp

    def query(self, using_master=False):
        if using_master:
            return self.using_master_session().query_property(BaseQuery)

        if self._slaves_session:
            return self.using_slave_session().query_property(BaseQuery)
        else:
            return self.using_master_session().query_property(BaseQuery)

    @async_execute
    def ping_db(self):
        # 防止数据库失联，定时ping一下
        ping = 'SELECT 0'
        try:
            self._master_session.execute(ping)
            self._master_session.remove()
            for slave in self._slaves_session:
                slave.execute(ping)
                slave.remove()
        except Exception, ex:
            SysLogger.error('ping mysql error:' + ex.message)

    def create_db(self):

        if not self._master_engine.echo:
            self._master_engine.echo = True
        Model.metadata.create_all(self._master_engine)

    def drop_db(self):
        if not self._master_engine.echo:
            self._master_engine.echo = True
        Model.metadata.drop_all(self._master_engine)


def get_connector(conn_class_):
    if issubclass(conn_class_, ConnBase):
        return _Connector(conn_class_)
    else:
        raise ConfigError('conn_class_ is wrong.')


Connector = get_connector(SQLAlchemy)


class Pagination(object):
    """
    Internal helper class returned by :meth:`BaseQuery.paginate`.  You
    can also construct it from any other SQLAlchemy query object if you are
    working with other libraries.  Additionally it is possible to pass `None`
    as query object in which case the :meth:`prev` and :meth:`next` will
    no longer work.
    """

    def __init__(self, query, page, per_page, total, items):
        # : the unlimited query object that was used to create this
        # : pagination object.
        self.query = query
        # : the current page number (1 indexed)
        self.page = page
        # : the number of items to be displayed on a page.
        self.per_page = per_page
        # : the total number of items matching the query
        self.total = total
        # : the items for the current page
        self.items = items

    @property
    def pages(self):
        """The total number of pages"""
        if self.per_page == 0:
            pages = 0
        else:
            pages = int(ceil(self.total / float(self.per_page)))
        return pages

    def prev(self, error_out=False):
        """Returns a :class:`Pagination` object for the previous page."""
        assert self.query is not None, 'a query object is required ' \
                                       'for this method to work'
        return self.query.paginate(self.page - 1, self.per_page, error_out)

    @property
    def prev_num(self):
        """Number of the previous page."""
        return self.page - 1

    @property
    def has_prev(self):
        """True if a previous page exists"""
        return self.page > 1

    def next(self, error_out=False):
        """Returns a :class:`Pagination` object for the next page."""
        assert self.query is not None, 'a query object is required ' \
                                       'for this method to work'
        return self.query.paginate(self.page + 1, self.per_page, error_out)

    @property
    def has_next(self):
        """True if a next page exists."""
        return self.page < self.pages

    @property
    def next_num(self):
        """Number of the next page"""
        return self.page + 1

    def iter_pages(self, left_edge=2, left_current=2,
                   right_current=5, right_edge=2):
        """Iterates over the page numbers in the pagination.  The four
        parameters control the thresholds how many numbers should be produced
        from the sides.  Skipped page numbers are represented as `None`.
        This is how you could render such a pagination in the templates:

        .. sourcecode:: html+jinja

            {% macro render_pagination(pagination, endpoint) %}
              <div class=pagination>
              {%- for page in pagination.iter_pages() %}
                {% if page %}
                  {% if page != pagination.page %}
                    <a href="{{ url_for(endpoint, page=page) }}">{{ page }}</a>
                  {% else %}
                    <strong>{{ page }}</strong>
                  {% endif %}
                {% else %}
                  <span class=ellipsis>…</span>
                {% endif %}
              {%- endfor %}
              </div>
            {% endmacro %}
        """
        last = 0
        for num in xrange(1, self.pages + 1):
            if num <= left_edge or \
                    (num > self.page - left_current - 1 and
                             num < self.page + right_current) or \
                            num > self.pages - right_edge:
                if last + 1 != num:
                    yield None
            yield num
            last = num



from setuptools import setup, find_packages
import torngas

setup(
    name="torngas",
    version=torngas.__version__,
    description="torngas based on tornado",
    long_description="torngas is based on tornado,django like web framework.",
    keywords='python torngas django tornado',
    author="mqingyn",
    url="https://github.com/mqingyn/torngas",
    license="BSD",
    packages=find_packages(),
    package_data={'torngas': ['resource/exception.html']},
    author_email="mqingyn@gmail.com",
    requires=['tornado', 'futures'],
    classifiers=[
        'Development Status :: 4 - Beta',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
    ],
    scripts=[],
    include_package_data=True,
    install_requires=[
        'futures',
    ],
)



#!/usr/bin/env python
# -*- coding: utf-8  -*-
"""
Created by qingyun.meng on 202014/9/4.
Modify by: qingyun.meng
Description: 
"""
import tornado.locale
from torngas.settings_manager import settings
from torngas.cache import close_caches


class HandlerMixin(object):
    _url_kwargs = {}

    def __init__(self, application, request, **kwargs):
        if kwargs:
            self._url_kwargs.update(kwargs)
            kwargs.clear()
        super(HandlerMixin, self).__init__(application, request, **kwargs)

    def prepare(self):
        res = self.application.middleware_fac.run_request(self)
        self.on_prepare()
        return res

    def on_prepare(self):
        pass

    def render_string(self, template_name, **kwargs):
        render_return = self.application.middleware_fac.run_render(self, template_name, **kwargs)
        self.application.middleware_fac.catch_middleware_exc(render_return)
        return super(HandlerMixin, self).render_string(template_name, **kwargs)

    def finish(self, chunk=None):
        # finish之前可能执行过多次write，反而chunk可能为None
        # 真正的chunk数据在self._write_buffer中，包含历次write的数据
        # 这里将chunk数据write进_write_buffer中，然后将chunk置空
        if chunk:
            self.write(chunk)
            chunk = None
        finish_return = self.application.middleware_fac.run_response(self, self._write_buffer)
        self.application.middleware_fac.catch_middleware_exc(finish_return)
        super(HandlerMixin, self).finish(chunk)

    def write(self, chunk, status=None):
        if status:
            self.set_status(status)
        super(HandlerMixin, self).write(chunk)

    def log_exception(self, typ, value, tb):
        """重写404请求的异常处理
        """
        if not self.application.middleware_fac.run_exception(self, typ, value, tb):
            super(HandlerMixin, self).log_exception(typ, value, tb)

    def on_finish(self):
        complate_return = self.application.middleware_fac.run_endcall(self)
        self.application.middleware_fac.catch_middleware_exc(complate_return)
        try:
            close_caches()
        except Exception, ex:
            pass
        self.complete_finish()

    def complete_finish(self):
        pass

    def get_user_locale(self):
        if settings.TRANSLATIONS_CONF.use_accept_language:
            return None

        return tornado.locale.get(settings.TRANSLATIONS_CONF.locale_default)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""
中间件可以实现的方法

class MyMiddleware(object):
    编写中间件需实现其中任何一个方法即可
    中间件的执行流程中在请求阶段，process_call,process_request，process_exception按照中间件的声明顺序执行，
    在响应过程中，process_response，process_render和process_endcall则是按声明顺序倒序执行）
    如果希望提前finish请求并终止当前中间件继续执行，在调用finish后需要return True
    :clear: 如果希望接下来的所有中间件流程终止，则在方法头部调用clear()，以清空中间件的执行队列


    def process_init(self, application):
        :param application: 应用程序对象，此方法在tornado启动时执行一次

    def process_call(self, request, clear):
        在请求进入application时调用，参数为请求对象，此时还未匹配路由
        您不能在此方法内finish请求
        :param request: 请求对象


    def process_request(self, handler, clear):
        匹配路由后，执行处理handler时调用
        :param handler: handler对象
        支持异步

    def process_render(self, handler, clear, template_name, **kwargs):
        此方法在调用render/render_string时发生


    def process_response(self, handler, clear, chunk):
        请求结束后响应时调用，此方法在render之后，finish之前执行，可以对chunk做最后的封装和处理
        :param handler: handler对象

    def process_endcall(self, handler, clear):
        :param handler: handler对象
        请求结束后调用，此时已完成响应并呈现用户，一般用来处理收尾操作，清理缓存对象，断开连接等
    def process_exception(self,handler, clear, typ, value, tb):
        异常处理
"""
from functools import partial
from tornado.util import import_object
from tornado.log import gen_log
from tornado import gen
from copy import copy

try:
    from tornado.concurrent import is_future
except ImportError:
    from ..utils import is_future

_INIT_LIST, _CALL_LIST, _REQUEST_LIST, _RENDER_LIST = [], [], [], []
_RESPONSE_LIST, _ENDCALL_LIST, _EXCEPTION_LIST, = [], [], []
_TINIT, _TCALL, _TREQ, _TREN, _TRES, _TEND, _TEXC = 1, 2, 3, 4, 5, 6, 7
_TYPES = (_TINIT, _TCALL, _TREQ, _TREN, _TRES, _TEND, _TEXC)


class Manager(object):
    _call_object = None

    def register(self, name):
        if isinstance(name, (str, unicode,)):
            name = import_object(name)
        name = name()

        if hasattr(name, 'process_init'):
            _INIT_LIST.append(name)
        if hasattr(name, 'process_call'):
            _CALL_LIST.insert(0, name)
        if hasattr(name, 'process_request'):
            _REQUEST_LIST.insert(0, name)
        if hasattr(name, 'process_render'):
            _RENDER_LIST.append(name)

        if hasattr(name, 'process_response'):
            _RESPONSE_LIST.append(name)

        if hasattr(name, 'process_endcall'):
            _ENDCALL_LIST.append(name)

        if hasattr(name, 'process_exception'):
            _EXCEPTION_LIST.insert(0, name)

    def register_all(self, names):
        if not names:
            names = ()

        for midd_class in names:
            self.register(midd_class)

    def set_request(self, request):
        request.call_midds = copy(_CALL_LIST)
        request.request_midds = copy(_REQUEST_LIST)
        request.render_midds = copy(_RENDER_LIST)
        request.response_midds = copy(_RESPONSE_LIST)
        request.end_midds = copy(_ENDCALL_LIST)
        request.exc_midds = copy(_EXCEPTION_LIST)

    def _get_func(self, request, m, func):
        try:
            cls = []
            if hasattr(request, m):
                cls = getattr(request, m)
            if len(cls):
                cls = cls.pop()

                return getattr(cls, func)
        except Exception, ex:
            gen_log.error(ex)

    @gen.coroutine
    def execute_next(self, request, types, process_object, *args, **kwargs):
        midd = None

        if types == _TCALL:
            midd = ('call_midds', 'process_call',)
        elif types == _TREQ:
            midd = ('request_midds', 'process_request',)
        elif types == _TREN:
            midd = ('render_midds', 'process_render',)
        elif types == _TRES:
            midd = ('response_midds', 'process_response',)
        elif types == _TEND:
            midd = ('end_midds', 'process_endcall',)
        elif types == _TEXC:
            midd = ('exc_midds', 'process_exception',)
        if midd:
            while 1:
                method = self._get_func(request, midd[0], midd[1])
                if method and callable(method):

                    clear = partial(self.clear_all, request)
                    result = method(process_object, clear, *args, **kwargs)
                    if is_future(result):
                        result = yield result
                        if result:
                            break
                else:
                    break

    def clear_all(self, request):
        request.call_midds = []
        request.request_midds = []
        request.render_midds = []
        request.response_midds = []
        request.end_midds = []

    def run_init(self, application):
        for func in _INIT_LIST:
            if callable(func.process_init):
                func.process_init(application)

    def run_call(self, request):
        return self.execute_next(request, _TCALL, request)

    def run_request(self, handler):
        return self.execute_next(handler.request, _TREQ, handler)

    def run_render(self, handler, template=None, **kwargs):
        return self.execute_next(handler.request, _TREN, handler, template, **kwargs)

    def run_response(self, handler, chunk):
        return self.execute_next(handler.request, _TRES, handler, chunk)

    def run_endcall(self, handler):
        return self.execute_next(handler.request, _TEND, handler)

    def run_exception(self, handler, typ, value, tb):
        if _EXCEPTION_LIST:
            self.execute_next(handler.request, _TEXC, handler, typ, value, tb)
            return True

    def catch_middleware_exc(self, middle_result):
        exc_info = middle_result.exc_info()
        if exc_info:
            raise exc_info[1]





#!/usr/bin/env python
# -*- coding: utf-8 -*-
import traceback
import sys
import tornado
import httplib
import os
from tornado.web import RequestHandler
from tornado import template


class UncaughtExceptionMixin(object):
    def get_error_html(self, status_code, **kwargs):

        def get_snippet(fp, target_line, num_lines):
            if fp.endswith('.html'):
                fp = os.path.join(self.get_template_path(), fp)

            half_lines = (num_lines / 2)
            try:
                with open(fp) as f:
                    all_lines = [line for line in f]

                    return ''.join(all_lines[target_line - half_lines:target_line + half_lines])
            except Exception, ex:
                return ''

        if not self.application.settings.get('debug', False):
            if hasattr(self, 'exception_response'):
                # 如果handler存在此属性，将返回属性内容
                return self.exception_response
            full_message = kwargs.get('exception', None)
            if not full_message or unicode(full_message) == '':
                full_message = 'Server error.'

            return "<html><title>%(code)d: %(message)s</title><body>\
            <h1>%(code)d: %(message)s</h1>%(full_message)s</body></html>" % {
                "code": status_code,
                "message": httplib.responses[status_code],
                "full_message": full_message,
            }

        else:
            exception = kwargs.get('exception', None)
            resource = os.path.split(os.path.dirname(__file__))[0]
            tmpl_file = '/resource/exception.html'

            def render_string(template_name, **kw):
                template_path = self.get_template_path()
                if not template_path:
                    frame = sys._getframe(0)
                    web_file = frame.f_code.co_filename
                    while frame.f_code.co_filename == web_file:
                        frame = frame.f_back
                    template_path = os.path.dirname(frame.f_code.co_filename)
                with RequestHandler._template_loader_lock:
                    settings = self.application.settings
                    kwarg = {}

                    if "autoescape" in settings:
                        kwarg["autoescape"] = settings["autoescape"]
                    loader = template.Loader(template_path, **kwarg)
                t = loader.load(template_name)
                namespace = self.get_template_namespace()
                namespace.update(kw)
                return t.generate(**namespace)

            return render_string("%s%s" % (resource, tmpl_file),
                                 get_snippet=get_snippet,
                                 exception=exception,
                                 traceback=traceback,
                                 sys=sys, tornado=tornado,
                                 status_code=status_code, os=os,
                                 kwargs=kwargs)

    def write_error(self, status_code, **kwargs):

        if 'exc_info' in kwargs:
            exc_info = kwargs.pop('exc_info')
            kwargs['exception'] = exc_info[1]

        return self.finish(self.get_error_html(status_code, **kwargs))



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from tornado.escape import json_encode, json_decode


class FlashMessageMixIn(object):
    """
        Store a message between requests which the user needs to see.

        views
        -------

        self.flash("Welcome back, %s" % username, 'success')

        base.html
        ------------

        {% set messages = handler.get_flashed_messages() %}
        {% if messages %}
        <div id="flashed">
            {% for category, msg in messages %}
            <span class="flash-{{ category }}">{{ msg }}</span>
            {% end %}
        </div>
        {% end %}
    """
    _flash_name = "__flhMsg"

    def flash(self, message, category='message'):
        messages = self.messages()
        messages.append((category, message))
        self.set_secure_cookie(self._flash_name, json_encode(messages))

    def messages(self):
        messages = self.get_secure_cookie(self._flash_name)
        messages = json_decode(messages) if messages else []
        return messages

    def get_flashed_messages(self):
        messages = self.messages()
        self.clear_cookie(self._flash_name)
        return messages
